/* Copyright (c) 2001, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/geometry/geometry_internal.h,v 1.3 2001/11/12 09:08:17 wex Exp $
 *
 * $Log: geometry_internal.h,v $
 * Revision 1.3  2001/11/12 09:08:17  wex
 * Added high level GEOMETRY structure
 *
 * Revision 1.2  2001/11/01 04:26:53  wex
 * Working with Python scene description and geometry shaders
 *
 * Revision 1.1  2001/10/26 15:48:30  wex
 * First working version
 *
 *
 */

#ifndef _GEOMETRY_INTERNAL_H_
#define _GEOMETRY_INTERNAL_H_

#include "geometry.h"

struct geometry {
    PyObject_HEAD
    
    SUBD *subd;
    PARTICLE *particle;
    PATCH *patch;
};
        
typedef struct subd_face {
    VERTEX *v0;
    VERTEX *v1;
    VERTEX *v2;
    VERTEX *v3;
    double weight[4];
} SUBD_FACE;

struct subd {
    int face_count;
    SUBD_FACE **face;
};

struct particle {
    VERTEX **v;
    int count;
    int allocated_count;
};

GRID *grid_create(int nu, int nv);

void subd_initialize(SUBD *subd);
void subd_delete(SUBD *subd);

void vertex_initialize(VERTEX *vtx, double px, double py, double pz, double qx,
    double qy, double qz, double u, double v);
void vertex_delete(VERTEX *v);

void geometry_initialize(GEOMETRY *geometry);
void geometry_delete(GEOMETRY *geometry);


#endif /* _GEOMETRY_INTERNAL_H_ */

