/* Copyright (c) 2001, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/geometry/grid.c,v 1.5 2001/11/16 09:23:03 wex Exp $
 *
 * $Log: grid.c,v $
 * Revision 1.5  2001/11/16 09:23:03  wex
 * #ifdef'ed out unused code
 *
 * Revision 1.4  2001/11/12 09:08:17  wex
 * Added high level GEOMETRY structure
 *
 * Revision 1.3  2001/11/01 04:26:53  wex
 * Working with Python scene description and geometry shaders
 *
 * Revision 1.2  2001/10/29 00:50:22  wex
 * Fixed -Wall warnings.
 *
 * Revision 1.1  2001/10/26 15:48:30  wex
 * First working version
 *
 *
 */

#include <float.h>

#include "geometry.h"

GRID *
grid_create(int ns, int nt)
{
    GRID *grid;

    grid = (GRID *)malloc(sizeof(GRID));
    if (grid == NULL) {
        GEOMETRY_errno = GEOMETRY_ERR_MEMORY;
        return NULL;
    }

    grid->ns = ns;
    grid->nt = nt;

    grid->p = (double (*)[3])malloc(ns * nt * 3 * sizeof(double));
    if (grid->p == NULL) {
        GEOMETRY_errno = GEOMETRY_ERR_MEMORY;
        free(grid);
        return NULL;
    }
    
    grid->q = (double (*)[3])malloc(ns * nt * 3 * sizeof(double));
    if (grid->q == NULL) {
        GEOMETRY_errno = GEOMETRY_ERR_MEMORY;
        free(grid);
        free(grid->p);
        return NULL;
    }

    grid->uv = (double (*)[2])malloc(ns * nt * 2 * sizeof(double));
    if (grid->uv == NULL) {
        GEOMETRY_errno = GEOMETRY_ERR_MEMORY;
        free(grid);
        free(grid->p);
        free(grid->q);
        return NULL;
    }
        
    return grid;
}

void
GRID_destroy(GRID *grid)
{
    if (grid->p != NULL) {
        free(grid->p);
        grid->p = NULL;
    }
    if (grid->q != NULL) {
        free(grid->q);
        grid->q = NULL;
    }
    if (grid->uv != NULL) {
        free(grid->uv);
        grid->uv = NULL;
    }

    free(grid);
}

#if 0
void
GRID_get_local_bbox(GRID *grid, VECTOR **min, VECTOR **max)
{
    int i, u, v;
    double xmin, ymin, zmin, xmax, ymax, zmax;

    xmin = DBL_MAX;
    ymin = DBL_MAX;
    zmin = DBL_MAX;
    
    xmax = -DBL_MAX;
    ymax = -DBL_MAX;
    zmax = -DBL_MAX;
    
    for (i = 0, v = 0; v < grid->nt; v++) {
        for (u = 0; u < grid->ns; u++, i++) {
            if (grid->p[i][0] < xmin) xmin = grid->p[i][0];
            if (grid->p[i][1] < ymin) ymin = grid->p[i][1];
            if (grid->p[i][2] < zmin) zmin = grid->p[i][2];

            if (grid->p[i][0] > xmax) xmax = grid->p[i][0];
            if (grid->p[i][1] > ymax) ymax = grid->p[i][1];
            if (grid->p[i][2] > zmax) zmax = grid->p[i][2];

            if (grid->q[i][0] < xmin) xmin = grid->q[i][0];
            if (grid->q[i][1] < ymin) ymin = grid->q[i][1];
            if (grid->q[i][2] < zmin) zmin = grid->q[i][2];

            if (grid->q[i][0] > xmax) xmax = grid->q[i][0];
            if (grid->q[i][1] > ymax) ymax = grid->q[i][1];
            if (grid->q[i][2] > zmax) zmax = grid->q[i][2];
        }
    }

    *min = VECTOR_create(xmin, ymin, zmin);
    *max = VECTOR_create(xmax, ymax, zmax);
    
}
#endif

void
GRID_get_screen_bbox(GRID *grid, MATRIX *l2s, double *xmin, double *ymin,
    double *xmax, double *ymax)
{
    int i, u, v;
    VECTOR p, s;

    *xmin = DBL_MAX;
    *ymin = DBL_MAX;
    
    *xmax = -DBL_MAX;
    *ymax = -DBL_MAX;
    
    for (i = 0, v = 0; v < grid->nt; v++) {
        for (u = 0; u < grid->ns; u++, i++) {
            V3_SET(p.v, grid->p[i][0], grid->p[i][1], grid->p[i][2]);
            VECTOR_project(&p, l2s, &s);
            
            if (s.v[0] < *xmin) *xmin = s.v[0];
            if (s.v[1] < *ymin) *ymin = s.v[1];

            if (s.v[0] > *xmax) *xmax = s.v[0];
            if (s.v[1] > *ymax) *ymax = s.v[1];

            V3_SET(p.v, grid->q[i][0], grid->q[i][1], grid->q[i][2]);
            VECTOR_project(&p, l2s, &s);
                        
            if (s.v[0] < *xmin) *xmin = s.v[0];
            if (s.v[1] < *ymin) *ymin = s.v[1];

            if (s.v[0] > *xmax) *xmax = s.v[0];
            if (s.v[1] > *ymax) *ymax = s.v[1];
        }
    }
}

    
