/* Copyright (c) 2001, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/image/image_format.c,v 1.1 2001/10/26 15:48:36 wex Exp $
 *
 * $Log: image_format.c,v $
 * Revision 1.1  2001/10/26 15:48:36  wex
 * First working version
 *
 *
 *
 */

#include <stdio.h>
#include <string.h>

#include "image_internal.h"
#include "image_format.h"

struct image_format {
    char *name;
    IMAGE_FORMAT *next;
    
    int extension_count;
    char **extension;

    int (*open)(IMAGE_FORMAT_OPEN_INFO *info);
    int (*read_tile)(IMAGE_FORMAT_READ_TILE_INFO *info);
    int (*write)(IMAGE_FORMAT_WRITE_INFO *info);
};

IMAGE_FORMAT *image_format_list = NULL;


static char *get_extension(char *filename);
static IMAGE_FORMAT *find_format(char *filename);



IMAGE_FORMAT *
image_format_create(char *name)
{
    IMAGE_FORMAT *format;

    format = (IMAGE_FORMAT *)malloc(sizeof(IMAGE_FORMAT));
    if (format == NULL) {
        IMAGE_errno = IMAGE_ERR_MEMORY;
        return NULL;
    }

    format->name = strdup(name);
    format->extension_count = 0;
    format->extension = NULL;
    format->open = NULL;
    format->read_tile = NULL;
    format->write = NULL;

    format->next = image_format_list;
    image_format_list = format;

    return format;
}

void
image_format_set_open_callback(IMAGE_FORMAT *format,
    int (*callback)(IMAGE_FORMAT_OPEN_INFO *))
{
    format->open = callback;
}

void
image_format_set_read_tile_callback(IMAGE_FORMAT *format,
    int (*callback)(IMAGE_FORMAT_READ_TILE_INFO *))
{
    format->read_tile = callback;
}

void
image_format_set_write_callback(IMAGE_FORMAT *format,
    int (*callback)(IMAGE_FORMAT_WRITE_INFO *))
{
    format->write = callback;
}

void
image_format_add_extension(IMAGE_FORMAT *format, char *extension)
{
    int i;

    i = format->extension_count;
    format->extension_count++;
    format->extension = (char **)realloc(format->extension,
        (i+1) * sizeof(char *));
    format->extension[i] = strdup(extension);
}

int
image_format_open_for_tiled_reading(IMAGE *image, char *filename)
{
    IMAGE_FORMAT *format;
    IMAGE_FORMAT_OPEN_INFO info;
    
    format = find_format(filename);
    if (format == NULL) {
        IMAGE_errno = IMAGE_ERR_READ_FORMAT;
        return -1;
    }

    info.image = image;
    info.filename = filename;

    return format->open(&info);
}

int
image_format_read_tile(IMAGE *image, int viewport[4])
{
    IMAGE_FORMAT_READ_TILE_INFO info;
    
    if (image->read_format == NULL) {
        IMAGE_errno = IMAGE_ERR_FILE_NOT_OPEN;
        return -1;
    }

    info.image = image;
    info.viewport[0] = viewport[0];
    info.viewport[1] = viewport[1];
    info.viewport[2] = viewport[2];
    info.viewport[3] = viewport[3];

    return image->read_format->read_tile(&info);
}

int
image_format_write(IMAGE *image, char *filename)
{
    IMAGE_FORMAT *format;
    IMAGE_FORMAT_WRITE_INFO info;

    format = find_format(filename);
    if (format == NULL) {
        IMAGE_errno = IMAGE_ERR_WRITE_FORMAT;
        return -1;
    }

    info.image = image;
    info.filename = filename;

    return format->write(&info);
}

static char *
get_extension(char *filename)
{
    char *dot;
    static char extension[64];  /* I can't wait for this to be a bug! */

    dot = strrchr(filename, '.');
    if (dot == NULL) return NULL;

    strcpy(extension, ++dot);

    return extension;
}

static IMAGE_FORMAT *
find_format(char *filename)
{
    int i;
    char *ext;
    IMAGE_FORMAT *format;

    ext = get_extension(filename);
    if (ext == NULL) return NULL;

    for (format = image_format_list; format != NULL; format = format->next) {
        for (i = 0; i < format->extension_count; i++) {
            /* perhaps this should do a looser comparison? */
            if (strcmp(ext, format->extension[i]) == 0) {
                return format;
            }
        }
    }

    return NULL;
}
