/* Copyright (c) 2001, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/image/image_internal.h,v 1.6 2001/12/08 23:25:24 wex Exp $
 *
 * $Log: image_internal.h,v $
 * Revision 1.6  2001/12/08 23:25:24  wex
 * Moved client_list and event_list into IMAGE structure
 *
 * Revision 1.5  2001/12/05 10:21:11  wex
 * Added notify and thread safe code, off-by-one bugs, bilerp
 *
 * Revision 1.4  2001/11/01 04:26:55  wex
 * Working with Python scene description and geometry shaders
 *
 * Revision 1.3  2001/10/29 00:50:24  wex
 * Fixed -Wall warnings.
 *
 * Revision 1.2  2001/10/26 15:48:36  wex
 * First working version
 *
 * Revision 1.1  2001/07/18 04:58:51  wex
 * Added
 *
 *
 */

#ifndef _IMAGE_INTERNAL_H_
#define _IMAGE_INTERNAL_H_

#include <Python.h>
#include <pthread.h>
#include <list.h>

#include "image.h"
#include "image_format.h"

/* returns the tile index given the tile's x & y coordinates */
#define IMAGE_TILE_SIZE  32     /* 32x32 pixels in each tile */
#define IMAGE_TILE_SHIFT 5      /* 2^5 = 32 */
#define IMAGE_TILE_BYTES (IMAGE_TILE_SIZE*IMAGE_TILE_SIZE*sizeof(float))

#define IMAGE_TILE_IDX(image, x, y) ((y)*(image)->x_tile_count + (x))

#define IMAGE_TILE_INDEX(image, vx, vy) \
    (IMAGE_TILE_IDX((image), ((vx)-(image)->viewport[0]) >> IMAGE_TILE_SHIFT, \
        ((vy)-(image)->viewport[1]) >> IMAGE_TILE_SHIFT))

#define IMAGE_PIXEL_INDEX(image, x, y) (((x) % IMAGE_TILE_SIZE) + \
    (((y) % IMAGE_TILE_SIZE) << IMAGE_TILE_SHIFT))

#define IMAGE_WIDTH(image) ((image)->viewport[2] - (image)->viewport[0] + 1)
#define IMAGE_HEIGHT(image) ((image)->viewport[3] - (image)->viewport[1] + 1)

/* all pointers are optional, but they will always be defined
   the same for all tiles in the image */
typedef struct image_tile {
    float       *r, *g, *b;                 /* color */
    float       *a, *z;                     /* alpha, depth */
} IMAGE_TILE;

struct image {
    PyObject_HEAD

    IMAGE_TILE      **tile;
    int             x_tile_count;           /* number of tiles in x & y */
    int             y_tile_count;           /*   computed from viewport+pad */
    int             viewport[4];            /* [x0,y0,  x1, y1] */

    char            is_color_defined;
    char            is_alpha_defined;
    char            is_depth_defined;

    IMAGE_FORMAT    *read_format;           /* valid when open for reading */

    pthread_mutex_t notify_mutex;           /* protect event list */
    pthread_mutex_t modify_mutex;           /* protect any get/set operation */

    LIST            *client_list;
    LIST            *event_list;
    
#if 0    
    MATRIX          *w2c, *c2s;             /* for depth maps only */
#endif    
};


IMAGE *image_alloc(void);
void image_initialize(IMAGE *image);
IMAGE_TILE *image_tile_create(IMAGE *image, int i);
void image_compute_tiling(IMAGE *image);
void image_destroy_tiles(IMAGE *image);
void image_tile_destroy(IMAGE_TILE *tile);
void image_delete(IMAGE *image);


#endif /* _IMAGE_INTERNAL_H_ */
