/* Copyright (c) 2001, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/image/image_rgba.c,v 1.2 2001/12/20 21:25:32 wex Exp $
 *
 * $Log: image_rgba.c,v $
 * Revision 1.2  2001/12/20 21:25:32  wex
 * Removed EXPORT in favor of .def file.
 *
 * Revision 1.1  2001/12/19 00:59:08  wex
 * Win32 support (EXPORT), added image_rgba.c and VC++ project files.
 *
 *
 */

#include "image_internal.h"


#define RED_SHIFT		0
#define GRN_SHIFT		8
#define BLU_SHIFT		16
#define ALP_SHIFT		24

#define RGBA(r,g,b,a)  \
   ((a) << ALP_SHIFT | (r) << RED_SHIFT | (g) << GRN_SHIFT | (b) << BLU_SHIFT)


static void
compute_8bit_color(IMAGE *image, int x, int y, unsigned long *rgba)
{
    double rf, gf, bf, af;
    unsigned char r, g, b, a;
    
    IMAGE_get_color(image, x, y, &rf, &gf, &bf);
    af = IMAGE_get_alpha(image, x, y);

    /* NOTE: apply appropriate gamma here */

    if (rf < 0) r = 0;
    else if (rf > 1) r = 255;
    else r = (unsigned char)(rf * 255.9);
    
    if (gf < 0) g = 0;
    else if (gf > 1) g = 255;
    else g = (unsigned char)(gf * 255.9);
    
    if (bf < 0) b = 0;
    else if (bf > 1) b = 255;
    else b = (unsigned char)(bf * 255.9);
    
    if (af < 0) a = 0;
    else if (af > 1) a = 255;
    else a = (unsigned char)(bf * 255.9);

    *rgba = RGBA(r, g, b, a);
}

void
IMAGE_rgba_update_viewport(IMAGE *image, unsigned long *rgba, 
    int x0, int y0, int x1, int y1)
{
    int i, x, y, w;

    w = image->viewport[2] - image->viewport[0] + 1;

    if (x0 < image->viewport[0]) x0 = image->viewport[0];
    if (x1 > image->viewport[2]) x1 = image->viewport[2];
    if (y0 < image->viewport[1]) y0 = image->viewport[1];
    if (y1 > image->viewport[3]) y1 = image->viewport[3];
    if (x1 < x0 || y1 < y0) return;  /* should never happen */
    
    for (y = y0; y <= y1; y++) {
        i = y * w + x0;
        
        for (x = x0; x <= x1; x++, i++) {
            compute_8bit_color(image, x, y, &rgba[i]);
        }
    }
}

unsigned long *
IMAGE_create_rgba(IMAGE *image)
{
    int x0, y0, x1, y1, n;
    unsigned long *rgba;
    
    IMAGE_get_viewport(image, &x0, &y0, &x1, &y1);
    n = (x1 - x0 + 1) * (y1 - y0 + 1);

    rgba = (unsigned long *)malloc(n * sizeof(unsigned long));
    if (rgba == NULL) {
        IMAGE_errno = IMAGE_ERR_MEMORY;
        return NULL;
    }
    
    memset(rgba, 0, n * sizeof(unsigned long)); 
    
	IMAGE_rgba_update_viewport(image, rgba, x0, y0, x1, y1);

    return rgba;
}
