/* Copyright (c) 2001, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/image/image_tile.c,v 1.5 2001/12/20 21:25:32 wex Exp $
 *
 * $Log: image_tile.c,v $
 * Revision 1.5  2001/12/20 21:25:32  wex
 * Removed EXPORT in favor of .def file.
 *
 * Revision 1.4  2001/12/05 10:21:11  wex
 * Added notify and thread safe code, off-by-one bugs, bilerp
 *
 * Revision 1.3  2001/11/01 04:26:55  wex
 * Working with Python scene description and geometry shaders
 *
 * Revision 1.2  2001/10/26 15:48:36  wex
 * First working version
 *
 * Revision 1.1  2001/07/18 04:58:51  wex
 * Added
 *
 *
 */

#include <assert.h>

#include "image_internal.h"

int image_tiles_alloced = 0;
int image_tiles_destroyed = 0;


void
image_compute_tiling(IMAGE *image)
{
	int i, nx, ny;
    int x0, y0, x1, y1;
	
	image_destroy_tiles(image);

    x0 = image->viewport[0];
    y0 = image->viewport[1];
    x1 = image->viewport[2];
    y1 = image->viewport[3];

	/* Empty viewport? */
    assert(x1 > x0 && y1 > y0);

	/* Determine tile count */
	nx = (x1 - x0 + 1) / IMAGE_TILE_SIZE;
	nx += ((x1 - x0 + 1) % IMAGE_TILE_SIZE) != 0;
	image->x_tile_count = nx;
	
	ny = (y1 - y0 + 1) / IMAGE_TILE_SIZE;
	ny += ((y1 - y0 + 1) % IMAGE_TILE_SIZE) != 0;
	image->y_tile_count = ny;

	/* Allocate tiles */
	image->tile = (IMAGE_TILE **)malloc(nx * ny * sizeof(IMAGE_TILE *));
	for (i = 0; i < nx * ny; i++) {
		image->tile[i] = NULL;
	}
}

IMAGE_TILE *
image_tile_create(IMAGE *image, int i)
{
    IMAGE_TILE *tile;

    image_tiles_alloced++;
    
    assert(image->tile[i] == NULL);

    tile = (IMAGE_TILE *)malloc(sizeof(IMAGE_TILE));
    if (tile == NULL) {
        IMAGE_errno = IMAGE_ERR_MEMORY;
        return NULL;
    }

    if (image->is_color_defined) {
        tile->r = (float *)malloc(IMAGE_TILE_BYTES);
        if (tile->r == NULL) goto out_of_memory;
        
        tile->g = (float *)malloc(IMAGE_TILE_BYTES);
        if (tile->g == NULL) goto out_of_memory;

        tile->b = (float *)malloc(IMAGE_TILE_BYTES);
        if (tile->b == NULL) goto out_of_memory;

        memset(tile->r, 0, IMAGE_TILE_BYTES);
        memset(tile->g, 0, IMAGE_TILE_BYTES);
        memset(tile->b, 0, IMAGE_TILE_BYTES);
    } else {
        tile->r = NULL;
        tile->g = NULL;
        tile->b = NULL;
    }

    if (image->is_alpha_defined) {
        tile->a = (float *)malloc(IMAGE_TILE_BYTES);
        if (tile->a == NULL) goto out_of_memory;
        memset(tile->a, 0, IMAGE_TILE_BYTES);
    } else {
        tile->a = NULL;
    }
    
    if (image->is_depth_defined) {
        tile->z = (float *)malloc(IMAGE_TILE_BYTES);
        if (tile->z == NULL) goto out_of_memory;
        memset(tile->r, 0, IMAGE_TILE_BYTES);
    } else {
        tile->z = NULL;
    }
        
    image->tile[i] = tile;

    return tile;

 out_of_memory:

    IMAGE_errno = IMAGE_ERR_MEMORY;
    
    if (tile->r != NULL) free(tile->r);
    if (tile->g != NULL) free(tile->g);
    if (tile->b != NULL) free(tile->b);
    if (tile->a != NULL) free(tile->a);
    if (tile->z != NULL) free(tile->z);
    free(tile);
    
    return NULL;
}

void
image_tile_destroy(IMAGE_TILE *tile)
{
    image_tiles_destroyed++;
    
    if (tile->r != NULL) {
        free(tile->r);
        free(tile->g);
        free(tile->b);
    }

    if (tile->a != NULL) {
        free(tile->a);
    }

    if (tile->z != NULL) {
        free(tile->z);
    }

    free(tile);
}
