 /* Copyright (c) 2001, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/ip/ip.h,v 1.3 2001/12/08 23:28:04 wex Exp $
 *
 * $Log: ip.h,v $
 * Revision 1.3  2001/12/08 23:28:04  wex
 * Updated FreeType defines, alpha_from_lum, referencing fixes.
 *
 * Revision 1.2  2001/11/26 21:26:11  wex
 * Basic image processing engine for point to point operations
 *
 * Revision 1.1  2001/11/16 09:21:57  wex
 * Added
 *
 *
 */

#include <Python.h>

#include <image.h>

/* error return codes */
#define IP_ERR_NONE          0
#define IP_ERR_MEMORY        1

/* contains the error code if a function returns an error */
extern int IP_errno;

/* returns a user readable string for error messages */
char *IP_error_string(void);


/** FreeType font rendering **/

#ifdef IP_FREETYPE_ENABLED

#include <ft2build.h>
#include FT_FREETYPE_H

typedef struct ip_font {
    PyObject_HEAD
    FT_Face face;
} IP_FONT;

int IP_FONT_initialize(IP_FONT *font, char *fontname);
int IP_FONT_render_string(IP_FONT *font, IMAGE *image, int x, int y, char *s,
    double r, double g, double b);
void IP_FONT_set_size(IP_FONT *font, double points);
void IP_FONT_destroy(IP_FONT *font);

#endif /* IP_FREETYPE_ENABLED */

/** Image operation graphs **/

/*
  over = IM_OP_read("foo.ppm");
  under = IM_OP_fill(0.5, 0.5, 0.5, 1, 0);
  matte = IM_OP_matte(over, under);
  im = IM_OP_compute_image(matte, NULL);
  IMAGE_write("out.ppm");
*/

typedef struct ip_op IP_OP;
typedef enum {IP_QUALITY_LOW, IP_QUALITY_MED, IP_QUALITY_HI} IP_QUALITY;


IP_OP *IP_OP_read(char *filename);
IP_OP *IP_OP_image(IMAGE *im);
IP_OP *IP_OP_fill(double r, double g, double b, double a, double z);
IP_OP *IP_OP_gradient(double tl[5], double tr[5], double bl[5], double br[5]);
IP_OP *IP_OP_matte(IP_OP *over, IP_OP *under);
IP_OP *IP_OP_alpha_from_lum(IP_OP *op);
IP_OP *IP_OP_convolve(IP_OP *input, int dim, double *kernel);
/*  IP_OP *IP_OP_transform(MATRIX *mat); */

void IP_OP_destroy(IP_OP *op);
int IP_OP_get_input_count(IP_OP *op);
int IP_OP_set_input(IP_OP *op, int index, IP_OP *input);
char *IP_OP_get_input_name(IP_OP *op, int index);
void IP_OP_set_global_viewport(int viewport[4]);
IP_QUALITY IP_OP_set_global_quality(IP_QUALITY quality);

/* pass NULL viewport to compute output image based on input viewports */
IMAGE *IP_OP_compute_image(IP_OP *op, int viewport[4]);


/** MIP-maps **/
typedef struct ip_map IP_MAP;

IP_MAP *IP_MAP_create(IMAGE *image);
IP_MAP *IP_MAP_read(int fd);
void IP_MAP_destroy(IP_MAP *map);
int IP_MAP_write(IP_MAP *map);
int IP_MAP_sample(IP_MAP *map, double u, double v, double du, double dv);
