#include "ip_op.h"
#include "ip_internal.h"

typedef struct ip_gradient_data {
    int viewport[4];
    double tl[5];
    double tr[5];
    double bl[5];
    double br[5];
} IP_GRADIENT_DATA;

static int ip_gradient_compute(IP_COMPUTE_INFO *info);


IP_OP *
IP_OP_gradient(double tl[5], double tr[5], double bl[5], double br[5])
{
    IP_GRADIENT_DATA *data;

    data = (IP_GRADIENT_DATA *)malloc(sizeof(IP_GRADIENT_DATA));
    if (data == NULL) {
        IP_errno = IP_ERR_MEMORY;
        return NULL;
    }

    memcpy(data->tl, tl, 5 * sizeof(double));
    memcpy(data->tr, tr, 5 * sizeof(double));
    memcpy(data->bl, bl, 5 * sizeof(double));
    memcpy(data->br, br, 5 * sizeof(double));

    ip_op_get_global_viewport(data->viewport);
    
    return ip_op_create("gradient", ip_gradient_compute, NULL, data);
}

static int
ip_gradient_compute(IP_COMPUTE_INFO *info)
{
    IP_GRADIENT_DATA *data = info->user_data;
    double s, t;
    double top[5], bot[5];
    
    s = (double)(info->x - data->viewport[0]) /
        (data->viewport[2] - data->viewport[0] + 1);
    t = (double)(info->y - data->viewport[1]) /
        (data->viewport[3] - data->viewport[1] + 1);

    /* bilinearly interpolate color from corners */
    ip_lerp_color(s, data->tl, data->tr, top);
    ip_lerp_color(s, data->bl, data->br, bot);
    ip_lerp_color(t, bot, top, info->output);

    return 0;
}

    
