#include <assert.h>

#include "ip_op.h"

typedef struct ip_image_data {
    IMAGE *image;
} IP_IMAGE_DATA;

static int ip_image_compute(IP_COMPUTE_INFO *info);
static int ip_image_viewport(IP_VIEWPORT_INFO *info);
static void ip_image_destroy(IP_OP *op);


IP_OP *
IP_OP_image(IMAGE *im)
{
    IP_OP *op;
    IP_IMAGE_DATA *data;

    data = (IP_IMAGE_DATA *)malloc(sizeof(IP_IMAGE_DATA));
    if (data == NULL) {
        IP_errno = IP_ERR_MEMORY;
        return NULL;
    }

    Py_INCREF((PyObject *)im);
    data->image= im;
    
    op = ip_op_create("image", ip_image_compute, ip_image_viewport, data);
    ip_op_set_destroy_fp(op, ip_image_destroy);

    return op;
}

static int
ip_image_compute(IP_COMPUTE_INFO *info)
{
    IP_IMAGE_DATA *data = info->user_data;
    
    IMAGE_get_color(data->image, info->x, info->y,
        &info->output[0], &info->output[1], &info->output[2]);
    info->output[3] = IMAGE_get_alpha(data->image, info->x, info->y);
    info->output[4] = IMAGE_get_depth(data->image, info->x, info->y);

    return 0;
}

static int
ip_image_viewport(IP_VIEWPORT_INFO *info)
{
    IP_IMAGE_DATA *data = info->user_data;

    IMAGE_get_viewport(data->image, &info->viewport[0], &info->viewport[1],
        &info->viewport[2], &info->viewport[3]);

    return 0;
}

static void
ip_image_destroy(IP_OP *op)
{
    IP_IMAGE_DATA *data = op->user_data;

    assert(IMAGE_check(data->image));
    Py_DECREF((PyObject *)data->image);
    free(data);
}
