/* Copyright (c) 2001, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/ip/ip_op.h,v 1.2 2002/01/07 22:26:14 wex Exp $
 *
 * $Log: ip_op.h,v $
 * Revision 1.2  2002/01/07 22:26:14  wex
 * Port to Windows, destroy fp for cleaning up operations
 *
 * Revision 1.1  2001/11/26 21:26:11  wex
 * Basic image processing engine for point to point operations
 *
 *
 */

#ifndef _IP_OP_H_
#define _IP_OP_H_

#include "ip.h"

#define IP_MAX_INPUT    10

typedef struct ip_compute_info {

    /* Do _NOT_ modify any of these values */
    IP_OP *op;
    void *user_data;
    int x, y;           /* pixel being computed */
    int viewport[4];    /* total output viewport */
    double input[IP_MAX_INPUT][5]; /* input pixel values */

    /* _ONLY_ modify the output pixels */
    double *output;     /* guaranteed to exist as a double[5] */
} IP_COMPUTE_INFO;

typedef struct ip_viewport_info {
    IP_OP *op;
    void *user_data;
    int viewport[4];    /* set this as the output */
} IP_VIEWPORT_INFO;

typedef struct ip_input {
    char *name;
    IP_OP *op;
} IP_INPUT;

struct ip_op {
    PyObject_HEAD
    
    char *name;
    
    int input_count;
    IP_INPUT *input;

    int (*compute)(IP_COMPUTE_INFO *info);
    int (*viewport)(IP_VIEWPORT_INFO *info);
    void (*destroy)(IP_OP *op);
    
    void *user_data;
};

IP_OP *ip_op_create(char *name, int (*compute)(IP_COMPUTE_INFO *info),
    int (*viewport)(IP_VIEWPORT_INFO *info), void *user_data);
void ip_op_set_destroy_fp(IP_OP *op, void (*destroy)(IP_OP *op));
void ip_op_add_input(IP_OP *op, char *name, IP_OP *input);
void ip_op_get_global_viewport(int viewport[4]);


#endif /* _IP_OP_H_ */
