#include "ip_op.h"

typedef struct ip_read_data {
    char *filename;
    int error_status;
    IMAGE *image;
} IP_READ_DATA;

static int ip_read_compute(IP_COMPUTE_INFO *info);
static int ip_read_viewport(IP_VIEWPORT_INFO *info);
static int ip_read_open(IP_READ_DATA *data);
static void ip_read_destroy(IP_OP *op);

IP_OP *
IP_OP_read(char *filename)
{
    IP_OP *op;
    IP_READ_DATA *data;

    data = (IP_READ_DATA *)malloc(sizeof(IP_READ_DATA));
    if (data == NULL) {
        IP_errno = IP_ERR_MEMORY;
        return NULL;
    }

    data->filename = strdup(filename);
    data->error_status = IMAGE_ERR_NONE;
    data->image = NULL;
    
    op = ip_op_create("read", ip_read_compute, ip_read_viewport, data);
    ip_op_set_destroy_fp(op, ip_read_destroy);

    return op;
}

static int
ip_read_compute(IP_COMPUTE_INFO *info)
{
    IP_READ_DATA *data = info->user_data;

    if (ip_read_open(data) < 0) {
        return -1;
    }
    
    IMAGE_get_color(data->image, info->x, info->y,
        &info->output[0], &info->output[1], &info->output[2]);
    info->output[3] = IMAGE_get_alpha(data->image, info->x, info->y);
    info->output[4] = IMAGE_get_depth(data->image, info->x, info->y);

    return 0;
}

static int
ip_read_viewport(IP_VIEWPORT_INFO *info)
{
    IP_READ_DATA *data = info->user_data;

    if (ip_read_open(data) < 0) {
        return -1;
    }

    IMAGE_get_viewport(data->image, &info->viewport[0], &info->viewport[1],
        &info->viewport[2], &info->viewport[3]);

    return 0;
}

static int
ip_read_open(IP_READ_DATA *data)
{
    if (data->image == NULL && data->error_status == IMAGE_ERR_NONE) {
        /* open the image for reading */
        data->image = IMAGE_create();
        if (data->image == NULL) {
            data->error_status = IMAGE_errno;
            return -1;
        }

        if (IMAGE_read(data->image, data->filename) < 0) {
            data->error_status = IMAGE_errno;
            fprintf(stderr, "Cannot read image \"%s\": %s\n",
                data->filename, IMAGE_error_string());
            return -1;
        }
    }

    return 0;
}

static void
ip_read_destroy(IP_OP *op)
{
    IP_READ_DATA *data = op->user_data;

    free(data->filename);
    if (data->image != NULL) {
        IMAGE_destroy(data->image);
        data->image = NULL;
    }

    free(data);
}

