/* Copyright (c) 2001, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/list/list.h,v 1.4 2001/12/20 21:20:23 wex Exp $
 *
 * $Log: list.h,v $
 * Revision 1.4  2001/12/20 21:20:23  wex
 * Removed EXORTS in favor of .def file
 *
 * Revision 1.3  2001/12/19 00:56:32  wex
 * Added EXPORT keyword for Win32 DLL support.  Added VC++ project files.
 *
 * Revision 1.2  2001/11/13 11:45:18  wex
 * Added LIST_is_empty()
 *
 * Revision 1.1  2001/10/26 15:48:38  wex
 * First working version
 *
 *
 */

#ifndef _LIST_H_
#define _LIST_H_

/*
  Use this library by declaring a structure type for the list elements,
  and make sure to have a LIST_ELEMENT structure at the top of the struct.
  For example:

  typedef stuct int_list {
      LIST_ELEMENT list_element;
      int value;
  } INT_LIST;

  LIST *list = LIST_create();
  INT_LIST element;

  element.value = 10;
  LIST_append(&element);
*/

/* error return codes */
#define LIST_ERR_NONE             0
#define LIST_ERR_MEMORY           1

/* contains the error code if a function returns an error */
extern int LIST_errno;

/* returns a user readable string for error messages */
char *LIST_error_string(void);

typedef struct list LIST;

typedef struct list_element {
    struct list_element *next;
    struct list_element *prev;
} LIST_ELEMENT;

LIST *LIST_create(void);
void LIST_init(LIST *list);     /* for locally allocated lists */
void LIST_destroy(LIST *list);  /* frees each element */

void LIST_append(LIST *list, void *element);
void LIST_prepend(LIST *list, void *element);
void LIST_delete(LIST *list, void *element);
void LIST_insert_before(LIST *list, void *element,void *before_element);

void *LIST_next(void *element);
void *LIST_prev(void *element);

void *LIST_head(void *list);
void *LIST_tail(void *list);
void *LIST_pop(LIST *list);

int LIST_count(LIST *list);
int LIST_is_empty(LIST *list);

void LIST_apply(LIST *list,
    void (*apply)(void *element, void *user_data), void *user_data);

void LIST_sort(LIST *list,
    int (*compare)(void *element_a, void *element_b, void *user_data),
    void *user_data);

#endif /* _LIST_H_ */
