/* Copyright (c) 2001, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/matrix/matrix_invert.c,v 1.3 2001/10/29 00:50:27 wex Exp $
 *
 * $Log: matrix_invert.c,v $
 * Revision 1.3  2001/10/29 00:50:27  wex
 * Fixed -Wall warnings.
 *
 * Revision 1.2  2001/07/17 09:37:26  wex
 * Added glue
 *
 * Revision 1.1  2001/07/17 03:32:39  wex
 * Added
 *
 *
 */

#include <stdlib.h>
#include <math.h>

#include "matrix_internal.h"

static double matrix_3x3_determinant(double a1, double a2, double a3,
    double b1, double b2, double b3, double c1,double c2,double c3);
static double matrix_2x2_determinant(double a, double b, double c, double d);


/* Note:  The adjoint/determinant method for inverting matrices is not very
          stable.  It would be better to implement a partial pivot inverter */


double
MATRIX_determinant(MATRIX *m)
{
    double a1, a2, a3, a4, b1, b2, b3, b4, c1, c2, c3, c4, d1, d2, d3, d4;

    a1 = m->m[0][0];
    b1 = m->m[1][0];
    c1 = m->m[2][0];
    d1 = m->m[3][0];
    
    a2 = m->m[0][1];
    b2 = m->m[1][1];
    c2 = m->m[2][1];
    d2 = m->m[3][1];
    
    a3 = m->m[0][2];
    b3 = m->m[1][2];
    c3 = m->m[2][2];
    d3 = m->m[3][2];
    
    a4 = m->m[0][3];
    b4 = m->m[1][3];
    c4 = m->m[2][3];
    d4 = m->m[3][3];
    
    return a1 * matrix_3x3_determinant(b2, b3, b4, c2, c3, c4, d2, d3, d4) -
        b1 * matrix_3x3_determinant(a2, a3, a4, c2, c3, c4, d2, d3, d4) +
        c1 * matrix_3x3_determinant(a2, a3, a4, b2, b3, b4, d2, d3, d4) -
        d1 * matrix_3x3_determinant(a2, a3, a4, b2, b3, b4, c2, c3, c4);
}

static double
matrix_3x3_determinant(double a1, double a2, double a3,
    double b1, double b2, double b3, double c1,double c2,double c3)
{
    return a1 * matrix_2x2_determinant(b2, b3, c2, c3) -
        b1 * matrix_2x2_determinant(a2, a3, c2, c3) +
        c1 * matrix_2x2_determinant(a2, a3, b2, b3);
}

static double
matrix_2x2_determinant(double a, double b, double c, double d)
{
    return a * d - b * c;
}

void
MATRIX_adjoint(MATRIX *m, MATRIX *dst)
{
    double a1, a2, a3, a4, b1, b2, b3, b4, c1, c2, c3, c4, d1, d2, d3, d4;

    a1 = m->m[0][0];
    b1 = m->m[1][0];
    c1 = m->m[2][0];
    d1 = m->m[3][0];
    
    a2 = m->m[0][1];
    b2 = m->m[1][1];
    c2 = m->m[2][1];
    d2 = m->m[3][1];
    
    a3 = m->m[0][2];
    b3 = m->m[1][2];
    c3 = m->m[2][2];
    d3 = m->m[3][2];
    
    a4 = m->m[0][3];
    b4 = m->m[1][3];
    c4 = m->m[2][3];
    d4 = m->m[3][3];

    dst->m[0][0] =  matrix_3x3_determinant(b2, b3, b4, c2, c3, c4, d2, d3, d4);
    dst->m[0][1] = -matrix_3x3_determinant(a2, a3, a4, c2, c3, c4, d2, d3, d4);
    dst->m[0][2] =  matrix_3x3_determinant(a2, a3, a4, b2, b3, b4, d2, d3, d4);
    dst->m[0][3] = -matrix_3x3_determinant(a2, a3, a4, b2, b3, b4, c2, c3, c4);

    dst->m[1][0] = -matrix_3x3_determinant(b1, b3, b4, c1, c3, c4, d1, d3, d4);
    dst->m[1][1] =  matrix_3x3_determinant(a1, a3, a4, c1, c3, c4, d1, d3, d4);
    dst->m[1][2] = -matrix_3x3_determinant(a1, a3, a4, b1, b3, b4, d1, d3, d4);
    dst->m[1][3] =  matrix_3x3_determinant(a1, a3, a4, b1, b3, b4, c1, c3, c4);

    dst->m[2][0] =  matrix_3x3_determinant(b1, b2, b4, c1, c2, c4, d1, d2, d4);
    dst->m[2][1] = -matrix_3x3_determinant(a1, a2, a4, c1, c2, c4, d1, d2, d4);
    dst->m[2][2] =  matrix_3x3_determinant(a1, a2, a4, b1, b2, b4, d1, d2, d4);
    dst->m[2][3] = -matrix_3x3_determinant(a1, a2, a4, b1, b2, b4, c1, c2, c4);

    dst->m[3][0] = -matrix_3x3_determinant(b1, b2, b3, c1, c2, c3, d1, d2, d3);
    dst->m[3][1] =  matrix_3x3_determinant(a1, a2, a3, c1, c2, c3, d1, d2, d3);
    dst->m[3][2] = -matrix_3x3_determinant(a1, a2, a3, b1, b2, b3, d1, d2, d3);
    dst->m[3][3] =  matrix_3x3_determinant(a1, a2, a3, b1, b2, b3, c1, c2, c3);
}

int
MATRIX_invert(MATRIX *m, MATRIX *dst)
{
    int i, j;
    double d;
    MATRIX tmp;

    if (dst == m) {
        MATRIX_COPY_DATA(tmp.m, m->m);
        m = &tmp;
    }
        
    d = MATRIX_determinant(m);
    if (fabs(d) < MATRIX_EPSILON) {
        MATRIX_errno = MATRIX_ERR_NOINVERSE;
        return -1;
    }

    MATRIX_adjoint(m, dst);
    d = 1 / d;
    
    for (i = 0; i < 4; i++) {
        for (j = 0; j < 4; j++) {
            dst->m[i][j] *= d;
        }
    }

    return 0;
}
