/* Copyright (c) 2001, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/matrix/matrix_xform.c,v 1.8 2001/12/20 21:22:26 wex Exp $
 *
 * $Log: matrix_xform.c,v $
 * Revision 1.8  2001/12/20 21:22:26  wex
 * Support for Win32
 *
 * Revision 1.7  2001/12/16 00:33:39  wex
 * Removed old code
 *
 * Revision 1.6  2001/11/10 05:58:16  wex
 * Added windowed projection
 *
 * Revision 1.5  2001/11/06 06:56:53  wex
 * Basic transformations fixed up
 *
 * Revision 1.4  2001/10/29 00:50:27  wex
 * Fixed -Wall warnings.
 *
 * Revision 1.3  2001/10/26 15:48:40  wex
 * First working version
 *
 * Revision 1.2  2001/07/17 09:37:26  wex
 * Added glue
 *
 * Revision 1.1  2001/07/17 03:32:39  wex
 * Added
 *
 *
 */

#include <math.h>

#include "matrix_internal.h"

void
MATRIX_set_identity(MATRIX *m)
{
    MATRIX_set_scale(m, 1, 1, 1);
}

void
MATRIX_set_scale(MATRIX *m, double x, double y, double z)
{
    m->m[0][0] = x; m->m[0][1] = 0; m->m[0][2] = 0; m->m[0][3] = 0;
    m->m[1][0] = 0; m->m[1][1] = y; m->m[1][2] = 0; m->m[1][3] = 0;
    m->m[2][0] = 0; m->m[2][1] = 0; m->m[2][2] = z; m->m[2][3] = 0;
    m->m[3][0] = 0; m->m[3][1] = 0; m->m[3][2] = 0; m->m[3][3] = 1;
}

void
MATRIX_set_translate(MATRIX *m, double x, double y, double z)
{
    m->m[0][0] = 1; m->m[0][1] = 0; m->m[0][2] = 0; m->m[0][3] = 0;
    m->m[1][0] = 0; m->m[1][1] = 1; m->m[1][2] = 0; m->m[1][3] = 0;
    m->m[2][0] = 0; m->m[2][1] = 0; m->m[2][2] = 1; m->m[2][3] = 0;
    m->m[3][0] = x; m->m[3][1] = y; m->m[3][2] = z; m->m[3][3] = 1;
}

/* Rotation order is around Z, then Y, then X */
void
MATRIX_set_rotate(MATRIX *m, double x, double y, double z)
{
    double cz, sz, cy, sy, cx, sx;

    cz = cos(z);
    sz = sin(z);
    cy = cos(y);
    sy = sin(y);
    cx = cos(x);
    sx = sin(x);

    m->m[0][0] = cz*cy;
    m->m[0][1] = sz*cx + cz*sy*sx;
    m->m[0][2] = sz*sx - cz*sy*cx;
    m->m[0][3] = 0;

    m->m[1][0] = -sz*cy;
    m->m[1][1] = cz*cx - sz*sy*sx;
    m->m[1][2] = cz*sx + sz*sy*cx;
    m->m[1][3] = 0;
    
    m->m[2][0] = sy;
    m->m[2][1] = -cy*sx;
    m->m[2][2] = cy*cx;
    m->m[2][3] = 0;

    m->m[3][0] = 0;
    m->m[3][1] = 0;
    m->m[3][2] = 0;
    m->m[3][3] = 1;
}

void
MATRIX_set_lens_projection(MATRIX *m, double fov, double hither, double yon)
{
	m->m[0][0] = cos(fov / 2) / sin(fov / 2);
	m->m[0][1] = 0;
	m->m[0][2] = 0;
	m->m[0][3] = 0;

	m->m[1][0] = 0;
	m->m[1][1] = cos(fov / 2) / sin(fov / 2);
	m->m[1][2] = 0;
	m->m[1][3] = 0;

	m->m[2][0] = 0;
	m->m[2][1] = 0;
	m->m[2][2] = -(yon + hither) / (yon - hither);
	m->m[2][3] = -1;

	m->m[3][0] = 0;
	m->m[3][1] = 0;
	m->m[3][2] = -(2 * yon * hither) / (yon - hither);
	m->m[3][3] = 0;
}


static void
matrix_set_multiply(MATRIX *m, MATRIX *a)
{
    MATRIX b;
    MATRIX_COPY_DATA(b.m, m->m);
    MATRIX_multiply(&b, a, m);
}

void
MATRIX_scale(MATRIX *m, double x, double y, double z)
{
    MATRIX s;

    MATRIX_set_scale(&s, x, y, z);
    matrix_set_multiply(m, &s);
}

void
MATRIX_translate(MATRIX *m, double x, double y, double z)
{
    MATRIX t;

    MATRIX_set_translate(&t, x, y, z);
    matrix_set_multiply(m, &t);
}

void
MATRIX_rotate(MATRIX *m, double x, double y, double z)
{
    MATRIX r;

    MATRIX_set_rotate(&r, x, y, z);
    matrix_set_multiply(m, &r);
}

void
MATRIX_multiply(MATRIX *a, MATRIX *b, MATRIX *dst)
{
    int i, j;
    MATRIX tmp;
    
    if (dst == a) {
        MATRIX_COPY_DATA(tmp.m, a->m);
        a = &tmp;
    } else if (dst == b) {
        MATRIX_COPY_DATA(tmp.m, b->m);
        b = &tmp;
    }
        
    for (i = 0; i < 4; i++) {
        for (j = 0; j < 4; j++) {
            dst->m[i][j] = a->m[i][0] * b->m[0][j] + a->m[i][1] * b->m[1][j] +
                a->m[i][2] * b->m[2][j] + a->m[i][3] * b->m[3][j];
        }
    }
}

void
MATRIX_lookat(MATRIX *m, double tx, double ty, double tz)
{
	double	px, py, pz;
	double	x[3], y[3], z[3];
	double	len;

	/*
	 * Pick out the translation, find viewing vector.
	 */
	px = m->m[3][0];
	py = m->m[3][1];
	pz = m->m[3][2];

	z[0] = tx - px;
	z[1] = ty - py;
	z[2] = tz - pz;

	/*
	 * Normalize z
	 */
	len = sqrt (z[0]*z[0] + z[1]*z[1] + z[2]*z[2]);

	if (len == 0) return;

	len = -1/len;  /* Negate since Z is towards viewer */

	z[0] *= len;
	z[1] *= len;
	z[2] *= len;

	/*
	 * Make up vector.
	 */
	y[0] = 0;
	y[1] = 1;
	y[2] = 0;

	/*
	 * x = y X z
	 */
	x[0] = y[1]*z[2] - z[1]*y[2];
	x[1] = y[2]*z[0] - z[2]*y[0];
	x[2] = y[0]*z[1] - z[0]*y[1];

	/*
	 * Normalize x
	 */
	len = sqrt (x[0]*x[0] + x[1]*x[1] + x[2]*x[2]);

	if (len == 0) return;

	len = 1/len;

	x[0] *= len;
	x[1] *= len;
	x[2] *= len;

	/*
	 * y = z X x
	 */
	y[0] = z[1]*x[2] - x[1]*z[2];
	y[1] = z[2]*x[0] - x[2]*z[0];
	y[2] = z[0]*x[1] - x[0]*z[1];

	/*
	 * Generate rotation matrix based on basis vectors and add original
	 * translation.  RIGHT handed space (transform this part for left-hand)
	 */

    m->m[0][0] = x[0]; m->m[0][1] = x[1]; m->m[0][2] = x[2]; m->m[0][3] = 0.0;
    m->m[1][0] = y[0]; m->m[1][1] = y[1]; m->m[1][2] = y[2]; m->m[1][3] = 0.0;
    m->m[2][0] = z[0]; m->m[2][1] = z[1]; m->m[2][2] = z[2]; m->m[2][3] = 0.0;
    m->m[3][0] = px;   m->m[3][1] = py;   m->m[3][2] = pz;   m->m[3][3] = 1.0;
}

void
MATRIX_set_window_projection(MATRIX *m, double window[4], double hither,
    double yon)
{
    m->m[0][0] = 2 * hither / (window[2] - window[0]);
    m->m[0][1] = 0;
    m->m[0][2] = (window[2] + window[0]) / (window[2] - window[0]);
    m->m[0][3] = 0;
    
    m->m[1][0] = 0;
    m->m[1][1] = 2 * hither / (window[3] - window[1]);
    m->m[1][2] = (window[3] + window[1]) / (window[3] - window[1]);
    m->m[1][3] = 0;

    m->m[2][0] = 0;
    m->m[2][1] = 0;
    m->m[2][2] = -1 * (yon + hither) / (yon - hither);
    m->m[2][3] = -2 * yon * hither / (yon - hither);
    
    m->m[3][0] = 0;
    m->m[3][1] = 0;
    m->m[3][2] = -1;
    m->m[3][3] = 0;
}

