/* Copyright (c) 2001, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/matrix/vector.c,v 1.7 2001/12/20 21:22:27 wex Exp $
 *
 * $Log: vector.c,v $
 * Revision 1.7  2001/12/20 21:22:27  wex
 * Support for Win32
 *
 * Revision 1.6  2001/11/16 09:23:42  wex
 * Removed get_data function
 *
 * Revision 1.5  2001/11/15 06:55:47  wex
 * Use Py_DECREF instead of free.
 *
 * Revision 1.4  2001/11/02 03:16:47  wex
 * Vector Python type and functions
 *
 * Revision 1.3  2001/11/01 04:26:56  wex
 * Working with Python scene description and geometry shaders
 *
 * Revision 1.2  2001/10/26 15:48:40  wex
 * First working version
 *
 * Revision 1.1  2001/07/17 03:32:39  wex
 * Added
 *
 *
 */

#include <stdlib.h>
#include <math.h>

#include "matrix_internal.h"

void
VECTOR_destroy(VECTOR *v)
{
    Py_DECREF(v);
}

void
VECTOR_set(VECTOR *v, double x, double y, double z)
{
    v->v[0] = x;
    v->v[1] = y;
    v->v[2] = z;
}

double
VECTOR_length(VECTOR *v)
{
    return sqrt(VECTOR_dot(v, v));
}

void
VECTOR_normalize(VECTOR *v)
{
    double d;

    d = VECTOR_length(v);
    if (d < MATRIX_EPSILON) {
        return;
    }

    d = 1/d;

    VECTOR_scale(v, d);
}

double
VECTOR_dot(VECTOR *a, VECTOR *b)
{
    return a->v[0] * b->v[0] + a->v[1] * b->v[1] + a->v[2] * b->v[2];
}

void
VECTOR_scale(VECTOR *a, double scale)
{
    a->v[0] *= scale;
    a->v[1] *= scale;
    a->v[2] *= scale;
}

void
VECTOR_cross(VECTOR *a, VECTOR *b, VECTOR *dst)
{
    double x, y, z;
    x = a->v[1] * b->v[2] - a->v[2] * b->v[1];
    y = a->v[2] * b->v[0] - a->v[0] * b->v[2];
    z = a->v[0] * b->v[1] - a->v[1] * b->v[0];
    V3_SET(dst->v, x, y, z);
}

double
VECTOR_x(VECTOR *v)
{
    return v->v[0];
}

double
VECTOR_y(VECTOR *v)
{
    return v->v[1];
}
    
double
VECTOR_z(VECTOR *v)
{
    return v->v[2];
}
    
void
VECTOR_add(VECTOR *a, VECTOR *b, VECTOR *dst)
{
    V3_SET(dst->v, a->v[0]+b->v[0], a->v[1]+b->v[1], a->v[2]+b->v[2]);
}

void
VECTOR_add_scalar(VECTOR *a, double s)
{
    a->v[0] += s;
    a->v[1] += s;
    a->v[2] += s;
}

void
VECTOR_print(VECTOR *v, FILE *fd)
{
    fprintf(fd, "Vector at %p = (%g, %g, %g)\n", v, v->v[0], v->v[1], v->v[2]);
}

