/* Copyright (c) 2001, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/matrix/vector_transform.c,v 1.3 2001/12/08 23:23:58 wex Exp $
 *
 * $Log: vector_transform.c,v $
 * Revision 1.3  2001/12/08 23:23:58  wex
 * Return w when projecting, added matrix copy.
 *
 * Revision 1.2  2001/11/01 04:26:56  wex
 * Working with Python scene description and geometry shaders
 *
 * Revision 1.1  2001/10/26 15:48:40  wex
 * First working version
 *
 *
 */

#include <math.h>

#include "matrix_internal.h"

void
VECTOR_transform(VECTOR *v, MATRIX *m, VECTOR *dst)
{
    double x, y, z;

	x = v->v[0]*m->m[0][0] + v->v[1]*m->m[1][0] +v->v[2]*m->m[2][0]+m->m[3][0];
	y = v->v[0]*m->m[0][1] + v->v[1]*m->m[1][1] +v->v[2]*m->m[2][1]+m->m[3][1];
    z = v->v[0]*m->m[0][2] + v->v[1]*m->m[1][2] +v->v[2]*m->m[2][2]+m->m[3][2];

    V3_SET(dst->v, x, y, z);
}

double
VECTOR_project(VECTOR *v, MATRIX *m, VECTOR *dst)
{
    double x, y, z, w;

	w = v->v[0]*m->m[0][3]+ v->v[1]*m->m[1][3] + v->v[2]*m->m[2][3]+m->m[3][3];
    if (w == 0) {
        V3_ZERO(dst->v);
        return 0;
    }

    x= (v->v[0]*m->m[0][0]+v->v[1]*m->m[1][0]+v->v[2]*m->m[2][0]+m->m[3][0])/w;
    y= (v->v[0]*m->m[0][1]+v->v[1]*m->m[1][1]+v->v[2]*m->m[2][1]+m->m[3][1])/w;
    z= (v->v[0]*m->m[0][2]+v->v[1]*m->m[1][2]+v->v[2]*m->m[2][2]+m->m[3][2])/w;

    V3_SET(dst->v, x, y, z);

    return w;
}
    
void
VECTOR_3x3_transform(VECTOR *v, MATRIX *m, VECTOR *dst)
{
    double x, y, z;

	x = m->m[0][0]*v->v[0] + m->m[0][1]*v->v[1] + m->m[0][2]*v->v[2];
	y = m->m[1][0]*v->v[0] + m->m[1][1]*v->v[1] + m->m[1][2]*v->v[2];
	z = m->m[2][0]*v->v[0] + m->m[2][1]*v->v[1] + m->m[2][2]*v->v[2];

    V3_SET(dst->v, x, y, z);
}
