/* Copyright (c) 2001, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/pool/pool.c,v 1.2 2002/01/03 05:17:19 wex Exp $
 *
 * $Log: pool.c,v $
 * Revision 1.2  2002/01/03 05:17:19  wex
 * Stub for initpool() required by distutils under windows
 *
 * Revision 1.1  2001/12/21 07:12:11  wex
 * Initial working version.
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "pool_internal.h"

#define POOL_BYTE_ALIGN     4

/* stub for linking under windows */
void initpool(void) {}


POOL *
POOL_create(int item_size, int items_per_block)
{
    POOL *pool;

    assert(item_size > 0);
    assert(items_per_block > 0);
    
    pool = (POOL *)malloc(sizeof(POOL));
    if (pool == NULL) {
        POOL_errno = POOL_ERR_MEMORY;
        return NULL;
    }

    /* must be large enough to hold a list node (4-bytes) */
    if (item_size < sizeof(POOL_NODE)) {
        item_size = sizeof(POOL_NODE);
    }
    
    /* make sure the size is aligned to 4-byte boundaries */
    if ((item_size % POOL_BYTE_ALIGN) != 0) {
        item_size += POOL_BYTE_ALIGN - (item_size % POOL_BYTE_ALIGN);
    }
    
    pool->item_size = item_size;
    pool->items_per_block = items_per_block;
    pool->block_list = LIST_create();
    pool->free_list = NULL;

    return pool;
}


void
POOL_destroy(POOL *pool)
{
    POOL_BLOCK *block;

    while ((block = LIST_pop(pool->block_list)) != NULL) {
        free(block->data);
        free(block);
    }

    LIST_destroy(pool->block_list);
    pool->block_list = NULL;    /* paranoia */

    free(pool);
}

void *
POOL_create_item(POOL *pool)
{
    POOL_NODE *item;
    POOL_BLOCK *block;

    /* free list? */
    if (pool->free_list != NULL) {
        item = pool->free_list;
        pool->free_list = item->next;
        return item;
    }

    /* current block? */
    block = LIST_tail(pool->block_list);
    if (block != NULL && block->count < pool->items_per_block) {
        unsigned char *tmp;
        tmp = block->data;
        tmp += block->count * pool->item_size;
/*        item = block->data + (block->count * pool->item_size) / 4;
          assert((unsigned char *)item == tmp); */
        ++block->count;
        return tmp;
    }
    
    /* add a new block */
    block = (POOL_BLOCK *)malloc(sizeof(POOL_BLOCK));
    if (block == NULL) {
        POOL_errno = POOL_ERR_MEMORY;
        return NULL;
    }

    block->data = malloc(pool->item_size * pool->items_per_block);
    if (block->data == NULL) {
        POOL_errno = POOL_ERR_MEMORY;
        return NULL;
    }

    block->count = 1;
    
    LIST_append(pool->block_list, block);

    return block->data;
}

void
POOL_destroy_item(POOL *pool, void *item)
{
    ((POOL_NODE *)item)->next = pool->free_list;
    pool->free_list = item;
}

