/* Copyright (c) 2001, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/shader/shader.c,v 1.7 2001/11/13 07:49:12 wex Exp $
 *
 * $Log: shader.c,v $
 * Revision 1.7  2001/11/13 07:49:12  wex
 * Python referencing cleanup, fixed leaks and bugs.
 *
 * Revision 1.6  2001/11/12 09:11:38  wex
 * Fixed up some memory leakage.
 *
 * Revision 1.5  2001/11/04 08:15:16  wex
 * Changes to support material bindings.
 *
 * Revision 1.4  2001/11/02 03:17:15  wex
 * Support for Python material and light shaders
 *
 * Revision 1.3  2001/11/01 04:26:57  wex
 * Working with Python scene description and geometry shaders
 *
 * Revision 1.2  2001/10/29 00:50:28  wex
 * Fixed -Wall warnings.
 *
 * Revision 1.1  2001/10/26 15:48:41  wex
 * First working version
 *
 *
 */

#include <stdio.h>
#include <string.h>

#include "shader_internal.h"



void
shader_initialize(SHADER *shader, SHADER_TYPE type, void *type_data)
{
    shader->parent = NULL;
    shader->type = type;

    switch (type) {
    case SHADER_TYPE_GEOMETRY:
        shader->shader.geometry = type_data;
        break;
    case SHADER_TYPE_MATERIAL:
        shader->shader.material = type_data;
        break;
    case SHADER_TYPE_LIGHT:
        shader->shader.light = type_data;
        break;
    case SHADER_TYPE_NONE:
        break;
    }

    shader->get_data_callback = NULL;
    shader->set_data_callback = NULL;
    shader->grid_capture = NULL;
    shader->grid_capture_data = NULL;
    shader->user_data = NULL;
    shader->cur_light = NULL;
    shader->data_list = LIST_create();
}

void
shader_delete(SHADER *shader)
{
    SHADER_DATA *data;
    
    if (shader->parent != NULL) {
        SHADER_destroy(shader->parent);
        shader->parent = NULL;
    }

    while ((data = LIST_pop(shader->data_list))) {
        free(data->name);
        free(data);
    }

    LIST_destroy(shader->data_list);
    shader->data_list = NULL;

    /*
    if (shader->type == SHADER_TYPE_MATERIAL) {
        abort();
        Py_DECREF(shader->shader.material);
    }
    */
}

void
SHADER_destroy(SHADER *shader)
{ 
    Py_DECREF(shader);
}
