/* Copyright (c) 2001, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/shader/shader_internal.h,v 1.6 2001/12/05 10:26:42 wex Exp $
 *
 * $Log: shader_internal.h,v $
 * Revision 1.6  2001/12/05 10:26:42  wex
 * Error and type checking
 *
 * Revision 1.5  2001/11/04 08:15:16  wex
 * Changes to support material bindings.
 *
 * Revision 1.4  2001/11/02 03:17:15  wex
 * Support for Python material and light shaders
 *
 * Revision 1.3  2001/11/01 04:26:57  wex
 * Working with Python scene description and geometry shaders
 *
 * Revision 1.2  2001/10/29 00:50:28  wex
 * Fixed -Wall warnings.
 *
 * Revision 1.1  2001/10/26 15:48:41  wex
 * First working version
 *
 *
 */

#ifndef _SHADER_INTERNAL_H_
#define _SHADER_INTERNAL_H_

#include <Python.h>

#include <list.h>
#include <geometry.h>

#include "shader.h"


typedef struct shader_binding {
    LIST_ELEMENT list_element;
    char *attribute_name;
    PyObject *map_class;
} SHADER_BINDING;

struct shader_geometry {
    PyObject *pobj;
    int (*bbox)(SHADER *shader, VECTOR *min, VECTOR *max);
    PyObject *generate;
};

struct shader_material {
    PyObject_HEAD

    LIST *binding_list;
    PyObject *pobj;
    PyObject *shade_pyobj_func;
    MATRIX *l2w, *w2l;
    int shade_initialized;
    int error;
};

struct shader_light {
    LIST_ELEMENT list_element;
    PyObject *pobj;
    PyObject *illuminate;
    MATRIX *l2w, *w2l;
};

typedef struct shader_data {
    LIST_ELEMENT list_element;
    char *name;
    SHADER_DATA_TYPE type;
    union {
        int i;
        double d;
        double v[3];
        char *s;
    } data;
} SHADER_DATA;

    
/* Thread specific structure for handling data callbacks */
struct shader {
    PyObject_HEAD
    
    SHADER *parent;
    SHADER_TYPE type;
    union {
        SHADER_GEOMETRY *geometry;
        SHADER_MATERIAL *material;
        SHADER_LIGHT *light;
    } shader;
    SHADER_DATA_CALLBACK get_data_callback;
    SHADER_DATA_CALLBACK set_data_callback;
    GRID_CAPTURE grid_capture;
    void *grid_capture_data;
    void *user_data;
    SHADER_LIGHT *cur_light;
    LIST *data_list;
};

SHADER *shader_create(SHADER *parent, SHADER_TYPE type, void *type_data);
void shader_initialize(SHADER *shader, SHADER_TYPE type, void *type_data);
void shader_destroy(SHADER *shader);
void shader_delete(SHADER *shader);
void shader_material_initialize(SHADER_MATERIAL *material);
void shader_material_delete(SHADER_MATERIAL *material);
int shader_material_binding_create(SHADER_MATERIAL *material, char *name,
    PyObject *map);
PyObject *shader_material_get_binding(SHADER_MATERIAL *material,
    char *name);
SHADER *shader_material_shade_child(PyObject *material, SHADER *parent);



#endif /* _SHADER_INTERNAL_H_ */
