/* Copyright (c) 2001, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/shader/shader_light.c,v 1.8 2001/12/08 23:32:42 wex Exp $
 *
 * $Log: shader_light.c,v $
 * Revision 1.8  2001/12/08 23:32:42  wex
 * Geometry viewports and SHADER_LIGHT_destroy
 *
 * Revision 1.7  2001/11/13 07:49:12  wex
 * Python referencing cleanup, fixed leaks and bugs.
 *
 * Revision 1.6  2001/11/03 07:42:51  wex
 * Updated setattr, error handling
 *
 * Revision 1.5  2001/11/02 19:57:13  wex
 * Check python object for shader data, handle tuples and vectors.
 *
 * Revision 1.4  2001/11/02 03:17:15  wex
 * Support for Python material and light shaders
 *
 * Revision 1.3  2001/11/01 04:26:57  wex
 * Working with Python scene description and geometry shaders
 *
 * Revision 1.2  2001/10/29 00:50:28  wex
 * Fixed -Wall warnings.
 *
 * Revision 1.1  2001/10/26 15:48:41  wex
 * First working version
 *
 *
 */

#include <stdio.h>
#include <assert.h>

#include "shader_internal.h"

/* global light list */
static LIST *shader_light_list = NULL;


SHADER_LIGHT *
SHADER_LIGHT_create(PyObject *pobj)
{
    SHADER_LIGHT *light;

    light = (SHADER_LIGHT *)malloc(sizeof(SHADER_LIGHT));
    if (light == NULL) {
        SHADER_errno = SHADER_ERR_MEMORY;
        return NULL;
    }

    light->pobj = pobj;
    light->illuminate = PyObject_GetAttrString(pobj, "illuminate");
    if (light->illuminate == NULL) {
        fprintf(stderr, "Cannot find light illuminate function\n");
    }

    if (shader_light_list == NULL) {
        shader_light_list = LIST_create();
    }

    LIST_append(shader_light_list, light);
    
    return light;
}

PyObject *
SHADER_get_light_list(SHADER *shader)
{
    int i, n;
    PyObject *list;
    SHADER_LIGHT *light;
    SHADER *cur;
    VECTOR rgb;
    PyObject *pargs, *pstatus;

    n = LIST_count(shader_light_list);
    list = PyList_New(n);
    light = LIST_head(shader_light_list);
    for (i = 0; i < n; i++) {
        cur = shader_create(shader, SHADER_TYPE_LIGHT, light);
        pargs = Py_BuildValue("(O)", cur);
        pstatus = PyEval_CallObject(light->illuminate, pargs);
        Py_DECREF(pargs); 
        if (PyErr_Occurred()) PyErr_Print();

        if (pstatus != NULL) {
            PyArg_Parse(pstatus, "(ddd)", &rgb.v[0], &rgb.v[1], &rgb.v[2]);
            Py_DECREF(pstatus);
        } else {
            rgb.v[0] = rgb.v[1] = rgb.v[2] = 0;
        }

        SHADER_set_vector(cur, "C", &rgb);
        
        PyList_SET_ITEM(list, i, (PyObject *)cur);
        
        light = LIST_next(light);
    }

    return list;
}

void
SHADER_LIGHT_destroy(SHADER_LIGHT *light)
{
    LIST_delete(shader_light_list, light);
    light->illuminate = NULL;
    light->pobj = NULL;
    free(light);
}

