/* Copyright (c) 2000, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/tui/tui_internal.h,v 1.17 2002/01/07 22:31:59 wex Exp $
 *
 * $Log: tui_internal.h,v $
 * Revision 1.17  2002/01/07 22:31:59  wex
 * Many fixes for both Windows and Linux, looped playback working
 *
 * Revision 1.16  2002/01/03 06:50:13  wex
 * Removed EXPORT, updated WIN32, handle some thread memory bugs.
 *
 * Revision 1.15  2001/12/19 01:01:44  wex
 * Support for Win32 including OpenGL and compiled as DLL.
 *
 * Revision 1.14  2001/12/16 00:32:58  wex
 * Updates for viewer, added checkmenuitem in Xlib.
 *
 * Revision 1.13  2001/12/05 10:47:01  wex
 * Changes to support image viewer in Python, compilation without warnings
 *
 * Revision 1.12  2000/11/04 18:00:08  wex
 * Initial support for Mac OSX.
 *
 * Revision 1.11  2000/10/06 03:48:46  wex
 * Slightly better events, widget windows capture other events.
 *
 * Revision 1.10  2000/10/05 10:29:41  wex
 * Added labels, titles, and textports.  Fixed parsing bug and radio look.
 *
 * Revision 1.9  2000/10/02 09:28:34  wex
 * Minor bug fixes.
 *
 * Revision 1.8  2000/10/02 05:56:00  wex
 * Added radio buttons, fixed minor bugs.
 *
 * Revision 1.7  2000/09/30 18:00:24  wex
 * Handle more visual types.  Menubars resize correctly.
 *
 * Revision 1.6  2000/09/30 15:59:14  wex
 * Saving and loading generated header files.
 *
 * Revision 1.5  2000/09/28 16:06:27  wex
 * Added TUI_APPLICATION, beginning layout and code output.
 *
 * Revision 1.4  2000/09/26 16:31:38  wex
 * Events working under Windows.
 *
 * Revision 1.3  2000/09/25 02:35:02  wex
 * Creating windows and widgets in win32, events still broken.
 *
 * Revision 1.2  2000/09/24 03:13:08  wex
 * Initial changes to support WIN32
 *
 * Revision 1.1.1.1  2000/09/16 23:38:08  wex
 * Initial revision
 *
 */

#ifndef _TUI_INTERNAL_H_
#define _TUI_INTERNAL_H_

#include <stdio.h>
#include <assert.h>

#include <list.h>

#include "tui.h"


#ifdef TUI_OSX
#include <HIToolbox/MacWindows.h>
#include <HIToolbox/Controls.h>
#include <HIToolbox/ControlDefinitions.h>
#include <CarbonCore/ToolUtils.h>
#endif



/* tui_menubar.h? */
#ifdef TUI_XWIN
#define TUI_MENUBAR_SPACING			26
#else
#define TUI_MENUBAR_SPACING			50
#endif

extern int tui_minor_version;
extern int tui_major_version;

typedef enum {
    TUI_GC_NULL = -1,

    /* Frame colors: */
    TUI_GC_BLACK = 0,
    TUI_GC_VERY_DARK_GRAY,
    TUI_GC_DARK_GRAY,
    TUI_GC_MEDIUM_GRAY,
    TUI_GC_LIGHT_GRAY,
    TUI_GC_MENU_GRAY,
    TUI_GC_VERY_LIGHT_GRAY,
    TUI_GC_WHITE,

    /* Generic colors: */
    TUI_GC_RED,
    TUI_GC_LIGHT_RED,
    TUI_GC_YELLOW,

    /* radio tag colors */
    TUI_GC_LIGHT_BLUE,
    TUI_GC_BLUE,

    /* textport */
    TUI_GC_TERRACOTTA,
    
    /* Special-purpose colors: */
    TUI_GC_CHECKMARK,
    TUI_GC_SHADOW, /* Shadow of checkback */

    /* Text: */
    TUI_GC_TEXT,
    TUI_GC_TEXT_BOLD,
    TUI_GC_TEXT_GRAY,
    TUI_GC_TEXT_MENUBAR,
    TUI_GC_TEXT_MENUBAR_GRAY,
    TUI_GC_TEXT_MENUITEM_GRAY,

    TUI_GC_MAX /* Must be last */
} TUI_GC;


typedef enum {
	TUI_COLOR_LIGHT_GRAY = 0,
	TUI_COLOR_MENU_GRAY,
	TUI_COLOR_MAX
} TUI_COLOR;

typedef struct tui_client {
    LIST_ELEMENT list_element;
    void (*callback)(TUI_CLIENT_INFO *info);
    void *user_data;
} TUI_CLIENT;

struct tui_application {
    LIST            *client_list;
    int             break_out_of_mainloop;
    
#ifdef TUI_XWIN
	Display 		*display;
    XVisualInfo     visual_info;
	GC				gc[TUI_GC_MAX];
	XFontStruct		*menu_font;
	XFontStruct		*text_font;
	XFontStruct		*title_font;
	unsigned long	color[TUI_COLOR_MAX];
#endif
#ifdef TUI_WIN32
	HINSTANCE		hInstance;
#endif
};


struct tui {
	TUI_WIDGET		*root;					/* children are windows */
    TUI_APPLICATION *app;
};


struct tui_widget {
	TUI				*tui;
	
	TUI_WIDGET_TYPE	type;
	
	TUI_WIDGET		*parent;
	TUI_WIDGET		*child;
	TUI_WIDGET		*next;

	int				value;
	int				x, y;
	int				w, h;
	char			*text;
	short			key;                    /* selected value for radio */
    
	unsigned char	visible;
	unsigned char	enabled;
	unsigned char	located;
	unsigned char	selected;

	void			(*callback)(TUI_WIDGET_INFO *info);
	void			*user_data;

    int             (*intercept)(TUI_EVENT_INFO *info);
    void            *intercept_user_data;
    
    char            *name;                  /* entry in output UI structure */
    char            *callback_name;         /* set in UI builder */
    
#ifdef TUI_XWIN	
	Window			window;
	GLXContext      glx_context;    /* only valid for OGL widgets */
	void			(*draw_fp)(TUI_WIDGET *widget);
	void			(*expose_fp)(TUI_WIDGET *widget, XExposeEvent *event);
	void			(*cross_fp)(TUI_WIDGET *widget, XCrossingEvent *event);
	void			(*button_fp)(TUI_WIDGET *widget, XButtonEvent *event);
	void			(*key_fp)(TUI_WIDGET *widget, XKeyEvent *event);
	void			(*motion_fp)(TUI_WIDGET *widget, XMotionEvent *event);
    void            (*resize_fp)(TUI_WIDGET *widget, int width, int height);
#endif

#ifdef TUI_WIN32
	int				control_id;
	HWND			hwnd;
	HMENU			hmenu;	    /* only valid if menu widget */
    HGLRC           hrc;
    HDC             hdc;
#endif

#ifdef TUI_OSX
    WindowRef       window;
    ControlRef      control;
    MenuRef         menu;
#endif    
};


TUI_WIDGET *tui_widget_allocate(void);
char *tui_widget_type_to_string(TUI_WIDGET_TYPE widget_type);
void tui_window_disable_matching_radios(TUI_WIDGET *window, TUI_WIDGET *radio);
int tui_pulldown_all_disabled(TUI_WIDGET *pulldown);
void tui_pulldown_open(TUI_WIDGET *pulldown);
void tui_pulldown_close(TUI_WIDGET *pulldown);
void tui_widget_append_to_list(TUI_WIDGET **head, TUI_WIDGET *widget);
void tui_widget_deselect_all_textports(TUI_WIDGET *widget);
void tui_widget_resize(TUI_WIDGET *widget, int w, int h);

void tui_oglwindow_force_redraw(TUI_WIDGET *widget);
void tui_redraw_all_oglwindows(TUI_WIDGET *root);

#ifdef TUI_XWIN
Window tui_create_xwin_window(TUI *tui, Window parent, int x, int y,
    int w, int h);
void tui_draw_frame(TUI *tui, Window window, int x, int y, int w, int h,
	TUI_GC *gc);
#endif

#ifdef TUI_WIN32
extern HINSTANCE tui_hInstance;
#define TUI_WINDOW_CLASS_NAME "tui_window_class"
HWND tui_create_win32_window(TUI *tui, int x, int y, int w, int h);
LRESULT CALLBACK tui_win32_wndproc(HWND hwnd, UINT message, WPARAM wparam,
	LPARAM lparam);
#endif


#endif  /* _TUI_INTERNAL_H_ */
