/* Copyright (c) 2000, Flarg Inc, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/tui/tui_label.c,v 1.5 2002/01/03 06:50:13 wex Exp $
 *
 * $Log: tui_label.c,v $
 * Revision 1.5  2002/01/03 06:50:13  wex
 * Removed EXPORT, updated WIN32, handle some thread memory bugs.
 *
 * Revision 1.4  2001/12/19 01:01:44  wex
 * Support for Win32 including OpenGL and compiled as DLL.
 *
 * Revision 1.3  2000/11/04 18:00:08  wex
 * Initial support for Mac OSX.
 *
 * Revision 1.2  2000/10/06 03:48:46  wex
 * Slightly better events, widget windows capture other events.
 *
 * Revision 1.1  2000/10/05 10:57:04  wex
 * Added.
 *
 *
 */

#include <string.h>

#include "tui_internal.h"

#define TUI_LABEL_HEIGHT        14

#ifdef TUI_XWIN
static void tui_label_draw(TUI_WIDGET *widget);
static void tui_label_expose(TUI_WIDGET *widget, XExposeEvent *event);
#endif



static TUI_WIDGET *
create_label(TUI_WIDGET *window, char *text, int x, int y,TUI_WIDGET_TYPE type)
{
    TUI_WIDGET *widget;
    int width;
    
#ifdef TUI_XWIN
    Window label_window;
    XFontStruct *font;

    if (type == TUI_LABEL) {
        font = window->tui->app->text_font;
    } else {
        font = window->tui->app->title_font;
    }
        
    width = XTextWidth(font, text, strlen(text));
    label_window = tui_create_xwin_window(window->tui, window->window, x, y,
        width, TUI_LABEL_HEIGHT);
    if (label_window == 0) {
        return NULL;
    }

    XSelectInput(window->tui->app->display, label_window,
        StructureNotifyMask | ExposureMask | ButtonPressMask |
        ButtonReleaseMask | PointerMotionMask);
#endif

#ifdef TUI_WIN32
	HWND hwnd;

	assert(window->hwnd != NULL);

    width = 100;
    
	hwnd = CreateWindow("static", text,
		WS_CHILD | WS_VISIBLE | SS_LEFTNOWORDWRAP, x, y, width, TUI_LABEL_HEIGHT,
		window->hwnd, (HMENU)-1, window->tui->app->hInstance, NULL);
	if (hwnd == NULL) {
		return NULL;
	}
#endif
	
    widget = tui_widget_allocate();
    if (widget == NULL) {
        return NULL;
    }

    widget->tui = window->tui;
    widget->type = type;
    widget->parent = window;
    tui_widget_append_to_list(&window->child, widget);
    
#ifdef TUI_XWIN
	widget->window = label_window;
    widget->draw_fp = tui_label_draw;
    widget->expose_fp = tui_label_expose;
#endif
    
	widget->x = x;
	widget->y = y;
	widget->w = width;
	widget->h = TUI_LABEL_HEIGHT;
	widget->text = strdup(text);

	TUI_WIDGET_set_visible(widget, 1);
	
	return widget;
}


TUI_WIDGET *
TUI_window_create_label(TUI_WIDGET *window, char *text, int x, int y)
{
    return create_label(window, text, x, y, TUI_LABEL);
}

TUI_WIDGET *
TUI_window_create_title(TUI_WIDGET *window, char *text, int x, int y)
{
    return create_label(window, text, x, y, TUI_TITLE);
}


#ifdef TUI_XWIN

static void
tui_label_draw(TUI_WIDGET *widget)
{
    TUI_GC gc_text;

    if (widget->type == TUI_LABEL) {
        gc_text = TUI_GC_TEXT;
    } else {
        gc_text = TUI_GC_TEXT_BOLD;
    }

    XDrawString(widget->tui->app->display, widget->window,
        widget->tui->app->gc[gc_text], 0, TUI_LABEL_HEIGHT,
        widget->text, strlen(widget->text));
}

static void
tui_label_expose(TUI_WIDGET *widget, XExposeEvent *event)
{
    widget->draw_fp(widget);
}

#endif /* TUI_XWIN */
