/* Copyright (c) 2000, Flarg Inc., All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/tui/tui_menubar.c,v 1.12 2002/01/03 06:50:13 wex Exp $
 *
 * $Log: tui_menubar.c,v $
 * Revision 1.12  2002/01/03 06:50:13  wex
 * Removed EXPORT, updated WIN32, handle some thread memory bugs.
 *
 * Revision 1.11  2001/12/19 01:01:44  wex
 * Support for Win32 including OpenGL and compiled as DLL.
 *
 * Revision 1.10  2001/12/05 10:47:01  wex
 * Changes to support image viewer in Python, compilation without warnings
 *
 * Revision 1.9  2000/10/05 10:29:41  wex
 * Added labels, titles, and textports.  Fixed parsing bug and radio look.
 *
 * Revision 1.8  2000/10/02 09:28:34  wex
 * Minor bug fixes.
 *
 * Revision 1.7  2000/10/02 05:56:00  wex
 * Added radio buttons, fixed minor bugs.
 *
 * Revision 1.6  2000/09/30 18:00:24  wex
 * Handle more visual types.  Menubars resize correctly.
 *
 * Revision 1.5  2000/09/28 16:06:27  wex
 * Added TUI_APPLICATION, beginning layout and code output.
 *
 * Revision 1.4  2000/09/26 16:31:38  wex
 * Events working under Windows.
 *
 * Revision 1.3  2000/09/25 02:35:02  wex
 * Creating windows and widgets in win32, events still broken.
 *
 * Revision 1.2  2000/09/24 03:13:08  wex
 * Initial changes to support WIN32
 *
 * Revision 1.1.1.1  2000/09/16 23:38:08  wex
 * Initial revision
 *
 */

#include <string.h>

#include "tui_internal.h"


#ifdef TUI_XWIN

#define TUI_MENUBAR_HEIGHT			26
#define TUI_MENUBAR_MARGIN			20
#define TUI_MENUBAR_BOTTOM_MARGIN	8

#define TUI_HELP_TITLE				"Help"


static void tui_menubar_draw(TUI_WIDGET *widget);
static void tui_menubar_expose(TUI_WIDGET *widget, XExposeEvent *event);
static void tui_menubar_button(TUI_WIDGET *widget, XButtonEvent *event);
static void tui_menubar_motion(TUI_WIDGET *widget, XMotionEvent *event);
static void tui_menubar_resize(TUI_WIDGET *widget, int w, int h);

#endif  /* TUI_XWIN */

TUI_WIDGET *
TUI_window_create_menubar(TUI_WIDGET *window)
{
	TUI_WIDGET *widget;
	
#ifdef TUI_XWIN	
	Window menubar_window;
	
	menubar_window = tui_create_xwin_window(window->tui, window->window,
		0, 0, window->w, TUI_MENUBAR_HEIGHT);
	if (menubar_window == 0) {
		return NULL;
	}

	XSelectInput(window->tui->app->display, menubar_window,
		ExposureMask | ButtonPressMask | ButtonReleaseMask | 
		PointerMotionMask | StructureNotifyMask);

#endif

#ifdef TUI_WIN32
	HMENU hmenu;

	hmenu = CreateMenu();

	if (hmenu == NULL) {
		return NULL;
	}
#endif	

	assert(window->type == TUI_WINDOW);

	widget = tui_widget_allocate();
	if (widget == NULL) {
		return NULL;
	}

	widget->tui = window->tui;
	widget->type = TUI_MENUBAR;

	widget->parent = window;
    tui_widget_append_to_list(&window->child, widget);

#ifdef TUI_XWIN	
	widget->window = menubar_window;
	widget->x = window->x;
	widget->y = window->y;
	widget->w = window->w;
	widget->h = TUI_MENUBAR_HEIGHT;

	widget->draw_fp = tui_menubar_draw;
	widget->expose_fp = tui_menubar_expose;
	widget->button_fp = tui_menubar_button;
	widget->motion_fp = tui_menubar_motion;
    widget->resize_fp = tui_menubar_resize;
#endif
	
#ifdef TUI_WIN32
	widget->hmenu = hmenu;
	assert(window->hwnd != NULL);
	SetMenu(window->hwnd, hmenu);
#endif
	
	widget->text = "menubar";
	
	TUI_WIDGET_set_enabled(widget, 1);
	TUI_WIDGET_set_visible(widget, 1);
	
	return widget;
}


#ifdef TUI_XWIN

static void
tui_menubar_draw(TUI_WIDGET *widget)
{
	int cur_x;
	GC gc_text;
	TUI_WIDGET *child;
	TUI_GC gc_neutral[] = {
        TUI_GC_DARK_GRAY,
        TUI_GC_WHITE,
        TUI_GC_VERY_LIGHT_GRAY,
        TUI_GC_VERY_LIGHT_GRAY,
        TUI_GC_LIGHT_GRAY,
        TUI_GC_MEDIUM_GRAY,
        TUI_GC_MEDIUM_GRAY,
        TUI_GC_DARK_GRAY,
        TUI_GC_VERY_DARK_GRAY,
        TUI_GC_NULL
    };
    TUI_GC gc_selected[] = {		/* modified slightly to better match FUI */
        TUI_GC_VERY_DARK_GRAY,
        TUI_GC_VERY_LIGHT_GRAY,
        TUI_GC_WHITE,
        TUI_GC_WHITE,
        TUI_GC_VERY_LIGHT_GRAY,
        TUI_GC_LIGHT_GRAY,
        TUI_GC_LIGHT_GRAY,
        TUI_GC_VERY_DARK_GRAY,
        TUI_GC_BLACK,
        TUI_GC_NULL
    };
	
	tui_draw_frame(widget->tui, widget->window, 0, 0, widget->w, widget->h,
		gc_neutral);

	cur_x = TUI_MENUBAR_MARGIN + TUI_MENUBAR_SPACING/2;
	
	for (child = widget->child; child != NULL; child = child->next) {

		if (strcmp(child->text, TUI_HELP_TITLE) == 0) {
			child->x = widget->w - TUI_MENUBAR_MARGIN -
				XTextWidth(widget->tui->app->menu_font, TUI_HELP_TITLE,
					strlen(TUI_HELP_TITLE));
		} else {
			child->x = cur_x - TUI_MENUBAR_SPACING/2;
		}

		child->w = XTextWidth(widget->tui->app->menu_font, child->text,
			strlen(child->text)) + TUI_MENUBAR_SPACING;

		if (child->selected) {
			tui_draw_frame(widget->tui, widget->window, child->x -
				TUI_MENUBAR_SPACING/2, 0, child->w, widget->h,gc_selected);
		}
		
		if (child->enabled && !tui_pulldown_all_disabled(child)) {
			gc_text = widget->tui->app->gc[TUI_GC_TEXT_MENUBAR];
		} else {
			gc_text = widget->tui->app->gc[TUI_GC_TEXT_MENUBAR_GRAY];
		}

		XDrawString(widget->tui->app->display, widget->window, gc_text,
            child->x, TUI_MENUBAR_HEIGHT - TUI_MENUBAR_BOTTOM_MARGIN,
			child->text, strlen(child->text));

		cur_x += child->w;
	}
}


static void
tui_menubar_expose(TUI_WIDGET *widget, XExposeEvent *event)
{
	tui_menubar_draw(widget);
}


static TUI_WIDGET *
find_pulldown_by_x(TUI_WIDGET *menubar, int x) 
{
	TUI_WIDGET *pulldown;

	for (pulldown=menubar->child; pulldown != NULL; pulldown=pulldown->next) {
		if (x > pulldown->x && x < pulldown->x + pulldown->w) {
			return pulldown;
		}
	}

	return NULL;
}


static TUI_WIDGET *
find_selected_child(TUI_WIDGET *widget)
{
	TUI_WIDGET *pulldown;
	
	for (pulldown = widget->child; pulldown != NULL; pulldown=pulldown->next) {
		if (pulldown->selected) {
			return pulldown;
		}
	}

	return NULL;
}


static void
tui_menubar_button(TUI_WIDGET *widget, XButtonEvent *event)
{
	TUI_WIDGET *pulldown, *menuitem;
	TUI_WIDGET_INFO info;
	
	assert(widget->type == TUI_MENUBAR);

	if (event->type == ButtonPress && event->button == 1) {
		pulldown = find_pulldown_by_x(widget, event->x);

		if (pulldown != NULL) {
			tui_pulldown_open(pulldown);
			tui_menubar_draw(widget);
		}
	}

	if (event->type == ButtonRelease && event->button == 1) {
		pulldown = find_selected_child(widget);
		if (pulldown != NULL) {
			menuitem = find_selected_child(pulldown);
			if (menuitem != NULL) {
                if (menuitem->callback != NULL) {
                    info.widget = menuitem;
                    info.user_data = menuitem->user_data;
                    menuitem->callback(&info);
                }
            
                menuitem->selected = 0;
			}

			tui_pulldown_close(pulldown);
			tui_menubar_draw(widget);
		}
	}
}


static void
tui_menubar_motion(TUI_WIDGET *widget, XMotionEvent *event)
{
	TUI_WIDGET *pulldown;
	TUI_WIDGET *active_pulldown;

	active_pulldown = find_selected_child(widget);
	if (active_pulldown == NULL) {
		return;
	}
	
	
	if (event->y >= 0 && event->y < TUI_MENUBAR_HEIGHT) {
		pulldown = find_pulldown_by_x(widget, event->x);

		if (pulldown != NULL) {
			if (active_pulldown != pulldown) {
				if (active_pulldown != NULL) {
					tui_pulldown_close(active_pulldown);
				}
				tui_pulldown_open(pulldown);
				tui_menubar_draw(widget);
			}
		}
	}

	/* Note: motion events for pulldowns happen in the menubar window
	   since X automatically performs a focus grab for the menubar
	   window since it captures both mouse down and mouse up events. */

	if (event->y >= TUI_MENUBAR_HEIGHT) {
		if (active_pulldown != NULL && active_pulldown->motion_fp != NULL) {
			active_pulldown->motion_fp(active_pulldown, event);
		}
	}
}

static void
tui_menubar_resize(TUI_WIDGET *widget, int width, int height)
{
    XResizeWindow(widget->tui->app->display, widget->window, width,
        TUI_MENUBAR_HEIGHT);
    widget->w = width;
}

    
#endif  /* TUI_XWIN */
