/* Copyright (c) 2000, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/tui/tui_menuitem.c,v 1.12 2002/01/03 06:50:13 wex Exp $
 *
 * $Log: tui_menuitem.c,v $
 * Revision 1.12  2002/01/03 06:50:13  wex
 * Removed EXPORT, updated WIN32, handle some thread memory bugs.
 *
 * Revision 1.11  2001/12/19 01:01:44  wex
 * Support for Win32 including OpenGL and compiled as DLL.
 *
 * Revision 1.10  2001/12/16 00:32:58  wex
 * Updates for viewer, added checkmenuitem in Xlib.
 *
 * Revision 1.9  2001/12/05 10:47:01  wex
 * Changes to support image viewer in Python, compilation without warnings
 *
 * Revision 1.8  2000/11/04 18:00:08  wex
 * Initial support for Mac OSX.
 *
 * Revision 1.7  2000/10/05 10:29:41  wex
 * Added labels, titles, and textports.  Fixed parsing bug and radio look.
 *
 * Revision 1.6  2000/10/02 05:56:00  wex
 * Added radio buttons, fixed minor bugs.
 *
 * Revision 1.5  2000/09/30 15:59:14  wex
 * Saving and loading generated header files.
 *
 * Revision 1.4  2000/09/26 16:31:38  wex
 * Events working under Windows.
 *
 * Revision 1.3  2000/09/25 02:35:02  wex
 * Creating windows and widgets in win32, events still broken.
 *
 * Revision 1.2  2000/09/24 03:13:08  wex
 * Initial changes to support WIN32
 *
 * Revision 1.1.1.1  2000/09/16 23:38:08  wex
 * Initial revision
 *
 */

#include <string.h>

#include "tui_internal.h"


TUI_WIDGET *
TUI_pulldown_create_menuitem(TUI_WIDGET *pulldown, char *text)
{
	TUI_WIDGET *widget;
#ifdef TUI_WIN32    
	static int cur_id = 1001;
#endif    
#ifdef TUI_OSX    
	unsigned char buf[255];
#endif    
	assert(pulldown->type == TUI_PULLDOWN);
	assert(text != NULL);
	assert(text[0] != '\0');

	widget = tui_widget_allocate();
	if (widget == NULL) {
		return NULL;
	}

	widget->tui = pulldown->tui;
	widget->type = TUI_MENUITEM;
	widget->parent = pulldown;
	widget->text = strdup(text);

#ifdef TUI_WIN32
	assert(pulldown->hmenu != NULL);
	if (!AppendMenu(pulldown->hmenu, MF_STRING,(UINT)cur_id,widget->text)){
		free(widget->text);
		free(widget);
		return NULL;
	}
	widget->control_id = cur_id;
	cur_id++;
	assert(pulldown->parent->parent->type == TUI_WINDOW);
	assert(pulldown->parent->parent->hwnd != NULL);
	DrawMenuBar(pulldown->parent->parent->hwnd);
#endif	

#ifdef TUI_OSX
    sprintf(buf, " %s", text);
    buf[0] = strlen(text);
    AppendMenuItemText(pulldown->menu, buf);
#endif
    
    tui_widget_append_to_list(&pulldown->child, widget);

	TUI_WIDGET_set_enabled(widget, 1);
	TUI_WIDGET_set_visible(widget, 1); 
    
	return widget;
}

	
TUI_WIDGET *
TUI_pulldown_create_separator(TUI_WIDGET *pulldown)
{
	TUI_WIDGET *widget, *prev;
	
	assert(pulldown->type == TUI_PULLDOWN);

	widget = tui_widget_allocate();
	if (widget == NULL) {
		return NULL;
	}

	widget->tui = pulldown->tui;
	widget->type = TUI_MENUITEM;
	widget->parent = pulldown;
	widget->text = "";

#ifdef TUI_WIN32
	assert(pulldown->hmenu != NULL);
	if (!AppendMenu(pulldown->hmenu, MF_SEPARATOR, 0, NULL)) {
		free(widget->text);
		free(widget);
		return NULL;
	}
#endif	
		
	/* append */
	if (pulldown->child == NULL) {
		pulldown->child = widget;
	} else {
		for (prev = pulldown->child; prev->next != NULL; prev = prev->next) {
			/*EMPTY*/
		}
		prev->next = widget;
	}


	return widget;
}


TUI_WIDGET *
TUI_pulldown_create_check_menuitem(TUI_WIDGET *pulldown, char *text)
{
    TUI_WIDGET *checkitem;

    checkitem = TUI_pulldown_create_menuitem(pulldown, text);
    checkitem->type = TUI_CHECK_MENUITEM;

    return checkitem;
}


void
TUI_menuitem_set_key(TUI_WIDGET *menuitem, char key)
{
    assert(menuitem->type == TUI_MENUITEM);
    menuitem->key = key;
}

