/* Copyright (c) 2000, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/tui/tui_misc.c,v 1.9 2001/12/19 01:01:44 wex Exp $
 *
 * $Log: tui_misc.c,v $
 * Revision 1.9  2001/12/19 01:01:44  wex
 * Support for Win32 including OpenGL and compiled as DLL.
 *
 * Revision 1.8  2000/10/02 09:28:34  wex
 * Minor bug fixes.
 *
 * Revision 1.7  2000/10/01 18:27:37  wex
 * Cleanups for Windows.
 *
 * Revision 1.6  2000/09/30 18:00:24  wex
 * Handle more visual types.  Menubars resize correctly.
 *
 * Revision 1.5  2000/09/30 15:59:14  wex
 * Saving and loading generated header files.
 *
 * Revision 1.4  2000/09/28 16:06:27  wex
 * Added TUI_APPLICATION, beginning layout and code output.
 *
 * Revision 1.3  2000/09/25 02:35:02  wex
 * Creating windows and widgets in win32, events still broken.
 *
 * Revision 1.2  2000/09/24 03:13:08  wex
 * Initial changes to support WIN32
 *
 * Revision 1.1.1.1  2000/09/16 23:38:08  wex
 * Initial revision
 *
 */


#include "tui_internal.h"


#ifdef TUI_WIN32
HWND
tui_create_win32_window(TUI *tui, int x, int y, int w, int h)
{
	return CreateWindow(TUI_WINDOW_CLASS_NAME, "Win32 Window",
		WS_OVERLAPPEDWINDOW, x, y, w, h,
		NULL, NULL, tui->app->hInstance, NULL);
}
#endif


#ifdef TUI_XWIN
Window
tui_create_xwin_window(TUI *tui, Window parent, int x, int y, int w, int h)
{
	Window window;
	XSetWindowAttributes attr;
    
	attr.background_pixel = tui->app->color[TUI_COLOR_LIGHT_GRAY];
	attr.do_not_propagate_mask = False;

	window = XCreateWindow(tui->app->display, parent, x, y, w, h, 0,
        tui->app->visual_info.depth, InputOutput,
        tui->app->visual_info.visual, CWBackPixel | CWDontPropagate, &attr);

	if (window == 0) {
		TUI_errno = TUI_ERR_WINDOW;
        fprintf(stderr, "Cannot create window\n");
		return 0;
	}

	return window;
}


void
tui_draw_frame(TUI *tui, Window window, int x, int y, int w, int h,TUI_GC *gc)
{
    int i, n;

    n = 0;
    while (gc[n] != TUI_GC_NULL) {
        n++;
    }

    for (i = 0; i < n/2; i++) {
        XDrawLine(tui->app->display, window, tui->app->gc[gc[i]],
            x + i, y + h - 1 - i, x + i, y + i);
        XDrawLine(tui->app->display, window, tui->app->gc[gc[i]],
            x + i, y + i, x + w - 1 - i, y + i);

        XDrawLine(tui->app->display, window, tui->app->gc[gc[n - 1 - i]],
            x + i + 1, y + h - 1 - i,
            x + w - 1 - i, y + h - 1 - i);
        XDrawLine(tui->app->display, window, tui->app->gc[gc[n - 1 - i]],
            x + w - 1 - i, y + h - 1 - i,
            x + w - 1 - i, y + i + 1);
    }

    if (n % 2 == 1) {
        XFillRectangle(tui->app->display, window, tui->app->gc[gc[n/2]],
            x + n/2, y + n/2, w - n + 1, h - n + 1);
    }
}

#endif  /* TUI_XWIN */
