/* Copyright (c) 2000, Flarg Inc, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/tui/tui_textport.c,v 1.6 2002/01/03 06:50:13 wex Exp $
 *
 * $Log: tui_textport.c,v $
 * Revision 1.6  2002/01/03 06:50:13  wex
 * Removed EXPORT, updated WIN32, handle some thread memory bugs.
 *
 * Revision 1.5  2001/12/19 01:01:44  wex
 * Support for Win32 including OpenGL and compiled as DLL.
 *
 * Revision 1.4  2001/12/05 10:47:01  wex
 * Changes to support image viewer in Python, compilation without warnings
 *
 * Revision 1.3  2000/11/04 18:00:08  wex
 * Initial support for Mac OSX.
 *
 * Revision 1.2  2000/10/06 03:48:46  wex
 * Slightly better events, widget windows capture other events.
 *
 * Revision 1.1  2000/10/05 10:57:04  wex
 * Added.
 *
 *
 */

#include <stdlib.h>
#include <string.h>

#include "tui_internal.h"

#define TUI_TEXTPORT_HEIGHT             28
#define TUI_TEXTPORT_LEFT_PAD           10
#define TUI_TEXTPORT_BOTTOM_PAD         9
#define TUI_TEXTPORT_CURSOR_TOP_PAD     6


#ifdef TUI_XWIN
#define XK_LATIN1
#define XK_MISCELLANY
#include <X11/keysymdef.h>
static void tui_textport_draw(TUI_WIDGET *widget);
static void tui_textport_expose(TUI_WIDGET *widget, XExposeEvent *event);
static void tui_textport_key(TUI_WIDGET *widget, XKeyEvent *event);
static void tui_textport_button(TUI_WIDGET *widget, XButtonEvent *event);
#endif



TUI_WIDGET *
TUI_window_create_textport(TUI_WIDGET *window, char *text,int x, int y, int w)
{
    TUI_WIDGET *widget;
    
#ifdef TUI_XWIN
    Window textport_window;

    textport_window = tui_create_xwin_window(window->tui, window->window, x, y,
        w, TUI_TEXTPORT_HEIGHT);
    if (textport_window == 0) {
        return NULL;
    }
    
    XSelectInput(window->tui->app->display, textport_window,
        StructureNotifyMask | ExposureMask | KeyPressMask | ButtonPressMask |
        PointerMotionMask | ButtonReleaseMask);
#endif

#ifdef TUI_WIN32
	HWND hwnd;

	assert(window->hwnd != NULL);

	hwnd = CreateWindow("edit", NULL,
		WS_CHILD | WS_VISIBLE | WS_BORDER | ES_LEFT | ES_AUTOHSCROLL, x, y, w, TUI_TEXTPORT_HEIGHT,
		window->hwnd, (HMENU)-1, window->tui->app->hInstance, NULL);
	if (hwnd == NULL) {
		return NULL;
	}
	SetWindowText(hwnd, text);
#endif
    widget = tui_widget_allocate();
    if (widget == NULL) {
        return NULL;
    }

    widget->tui = window->tui;
    widget->type = TUI_TEXTPORT;
    widget->parent = window;
    tui_widget_append_to_list(&window->child, widget);
    
#ifdef TUI_XWIN
	widget->window = textport_window;
    widget->draw_fp = tui_textport_draw;
    widget->expose_fp = tui_textport_expose;
    widget->key_fp = tui_textport_key;
    widget->button_fp = tui_textport_button;
#endif
    
	widget->x = x;
	widget->y = y;
	widget->w = w;
	widget->h = TUI_TEXTPORT_HEIGHT;
	widget->text = strdup(text);

	TUI_WIDGET_set_visible(widget, 1);

#ifdef TUI_XWIN
	tui_widget_deselect_all_textports(window->child);
    widget->selected = 1;
#endif

	return widget;
}


#ifdef TUI_XWIN

static void
tui_textport_draw(TUI_WIDGET *widget)
{
    int x, w, i, len;
    XRectangle rectangle;
    TUI_GC gc[] = {
        TUI_GC_LIGHT_GRAY,
        TUI_GC_MEDIUM_GRAY,
        TUI_GC_VERY_DARK_GRAY,
        TUI_GC_LIGHT_GRAY,
        TUI_GC_TERRACOTTA,
        TUI_GC_MEDIUM_GRAY,
        TUI_GC_VERY_DARK_GRAY,
        TUI_GC_LIGHT_GRAY,
        TUI_GC_VERY_LIGHT_GRAY,
        TUI_GC_NULL
    };

    tui_draw_frame(widget->tui, widget->window, 0, 0, widget->w, widget->h,gc);

    /* set the clip rectangle so we can display a bit of text just to
       the left and right of the cursor if characters are off to the
       left or right of the window without messing up the frame */
    rectangle.x = TUI_TEXTPORT_LEFT_PAD - 3;
    rectangle.y = 0;
    rectangle.width = widget->w - 2 * TUI_TEXTPORT_LEFT_PAD + 8;
    rectangle.height = TUI_TEXTPORT_HEIGHT;
    
    XSetClipRectangles(widget->tui->app->display,
        widget->tui->app->gc[TUI_GC_TEXT], 0, 0, &rectangle, 1, Unsorted);
    
    /* draw each character one at a time from the leftmost visible char */
    len = strlen(widget->text);
    x = TUI_TEXTPORT_LEFT_PAD + widget->key;

    for (i = 0; i < len; i++) {
        w = XTextWidth(widget->tui->app->text_font, &widget->text[i], 1);
        if (x+w > widget->w - TUI_TEXTPORT_LEFT_PAD + 10) break;

        if (x >= TUI_TEXTPORT_LEFT_PAD - 10) {
            XDrawString(widget->tui->app->display, widget->window,
                widget->tui->app->gc[TUI_GC_TEXT], x,
                TUI_TEXTPORT_HEIGHT - TUI_TEXTPORT_BOTTOM_PAD,
                &widget->text[i], 1);
        }

        if (widget->selected && i == widget->value) {
            XDrawLine(widget->tui->app->display, widget->window,
                widget->tui->app->gc[TUI_GC_BLUE],
                x, TUI_TEXTPORT_CURSOR_TOP_PAD,
                x, TUI_TEXTPORT_HEIGHT - TUI_TEXTPORT_CURSOR_TOP_PAD);
            XDrawLine(widget->tui->app->display, widget->window,
                widget->tui->app->gc[TUI_GC_BLUE],
                x+1, TUI_TEXTPORT_CURSOR_TOP_PAD,
                x+1, TUI_TEXTPORT_HEIGHT - TUI_TEXTPORT_CURSOR_TOP_PAD);
        }
        
        x += w;
    }

    XSetClipMask(widget->tui->app->display, widget->tui->app->gc[TUI_GC_TEXT],
        None);
    
    /* cursor at end of string */
    if (widget->selected && i == widget->value) {
        XDrawLine(widget->tui->app->display, widget->window,
            widget->tui->app->gc[TUI_GC_BLUE],
            x, TUI_TEXTPORT_CURSOR_TOP_PAD,
            x, TUI_TEXTPORT_HEIGHT - TUI_TEXTPORT_CURSOR_TOP_PAD);
        XDrawLine(widget->tui->app->display, widget->window,
            widget->tui->app->gc[TUI_GC_BLUE],
            x+1, TUI_TEXTPORT_CURSOR_TOP_PAD,
            x+1, TUI_TEXTPORT_HEIGHT - TUI_TEXTPORT_CURSOR_TOP_PAD);
    }
}

static void
tui_textport_expose(TUI_WIDGET *widget, XExposeEvent *event)
{
    widget->draw_fp(widget);
}


static void
tui_textport_button(TUI_WIDGET *widget, XButtonEvent *event)
{
    if (!widget->selected) {
        tui_widget_deselect_all_textports(widget->parent->child);
        widget->selected = 1;
        widget->draw_fp(widget);
    }
}


static void
tui_textport_key(TUI_WIDGET *widget, XKeyEvent *event)
{
    int len, w, i;
    char buffer[128], key, *text;
    KeySym keysym;
    XComposeStatus status;
    TUI_WIDGET_INFO info;

    if (event->type != KeyPress || (event->state & ControlMask)) {
        return;
    }
    
    XLookupString(event, buffer, 128, &keysym, &status);

    if (keysym == XK_Tab || !widget->selected) {
        /* forward back up to the window */
        widget->parent->key_fp(widget->parent, event);
        return;
    } else if (keysym == XK_BackSpace || keysym == XK_Delete) {
        if (widget->text != NULL && widget->value > 0) {
            widget->value -= 1;
            len = strlen(widget->text);
            text = (char *)malloc(len);
            if (widget->value > 0) {
                strncpy(text, widget->text, widget->value);
            }

            if (widget->value < len) {
                strncpy(&text[widget->value], &widget->text[widget->value+1],
                    len - widget->value);
                assert(text[len-1] == '\0');
            } else {
                widget->text[len-1] = '\0';
            }

            free(widget->text);
            widget->text = text;
            
            w = 0;
            for (i = 0; i < widget->value; i++) {
                w += XTextWidth(widget->tui->app->text_font,
                    &widget->text[i], 1);
            }

            if (w + widget->key < 0) {
                widget->key += widget->w - 2*TUI_TEXTPORT_LEFT_PAD;
                if (widget->key > 0) widget->key = 0;
            }
        }
    } else if (keysym == XK_Return) {
        if (widget->callback != NULL) {
            info.widget = widget;
            info.user_data = widget->user_data;
            widget->callback(&info);
        }
    } else if (keysym == XK_Left) {
        if (widget->value > 0) {
            /* check to see if we scroll right by finding the x location of
               the cursor and seeing if it is out of the viewable window */
            assert(widget->text != NULL);

            widget->value -= 1;
            
            w = 0;
            for (i = 0; i < widget->value; i++) {
                w += XTextWidth(widget->tui->app->text_font,
                    &widget->text[i], 1);
            }

            if (w + widget->key < 0) {
                widget->key -= w + widget->key;
                assert(widget->key <= 0);
            }
        }
    } else if (keysym == XK_Right) {
        if (widget->text != NULL) {
            len = strlen(widget->text);
            if (widget->value < len) {
                widget->value += 1;

                /* check to see if we scroll left by checking to see how
                   far the cursor is past the right edge of the window */
                w = 0;
                for (i = 0; i < widget->value; i++) {
                    w += XTextWidth(widget->tui->app->text_font,
                        &widget->text[i], 1);
                }

                if (w + widget->key > widget->w - 2*TUI_TEXTPORT_LEFT_PAD) {
                    widget->key -= w + widget->key -
                        (widget->w - 2*TUI_TEXTPORT_LEFT_PAD);
                }
            }
        }
    } else if (keysym < XK_space || keysym > XK_asciitilde) {
        return;
    } else {
        key = (char) ((int) keysym - XK_0 + '0');
        
        if (widget->text == NULL) {
            len = 0;
        } else {
            len = strlen(widget->text);
        }
    
        text = (char *)malloc(len + 2);

        if (widget->text != NULL) {
            if (widget->value > 0) {
                strncpy(text, widget->text, widget->value);
            }

            if (widget->value < len) {
                strncpy(&text[widget->value+1], &widget->text[widget->value],
                    len - widget->value);
            }
        }

        text[widget->value] = key;
        text[len+1] = '\0';
    
        free(widget->text);
        widget->text = text;

        widget->value += 1;
        w = 0;
        for (i = 0; i < widget->value; i++) {
            w += XTextWidth(widget->tui->app->text_font, &widget->text[i], 1);
        }

        if (w + widget->key > widget->w - 2*TUI_TEXTPORT_LEFT_PAD) {
            widget->key -= w + widget->key -
                (widget->w - 2*TUI_TEXTPORT_LEFT_PAD);
        }
    }
    
    widget->draw_fp(widget);
}

#endif /* TUI_XWIN */
