/* Copyright (c) 2000, Flarg Inc, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/tui/tui_window.c,v 1.14 2002/01/07 22:31:59 wex Exp $
 *
 * $Log: tui_window.c,v $
 * Revision 1.14  2002/01/07 22:31:59  wex
 * Many fixes for both Windows and Linux, looped playback working
 *
 * Revision 1.13  2001/12/19 01:01:44  wex
 * Support for Win32 including OpenGL and compiled as DLL.
 *
 * Revision 1.12  2001/12/05 10:47:01  wex
 * Changes to support image viewer in Python, compilation without warnings
 *
 * Revision 1.11  2000/11/04 18:00:08  wex
 * Initial support for Mac OSX.
 *
 * Revision 1.10  2000/10/06 03:48:46  wex
 * Slightly better events, widget windows capture other events.
 *
 * Revision 1.9  2000/10/05 10:29:41  wex
 * Added labels, titles, and textports.  Fixed parsing bug and radio look.
 *
 * Revision 1.8  2000/10/02 09:28:34  wex
 * Minor bug fixes.
 *
 * Revision 1.7  2000/10/02 07:16:48  wex
 * Keyboard equivalents.
 *
 * Revision 1.6  2000/10/02 05:56:00  wex
 * Added radio buttons, fixed minor bugs.
 *
 * Revision 1.5  2000/09/30 18:00:24  wex
 * Handle more visual types.  Menubars resize correctly.
 *
 * Revision 1.4  2000/09/30 15:59:14  wex
 * Saving and loading generated header files.
 *
 * Revision 1.3  2000/09/28 16:06:27  wex
 * Added TUI_APPLICATION, beginning layout and code output.
 *
 * Revision 1.2  2000/09/24 03:13:08  wex
 * Initial changes to support WIN32
 *
 * Revision 1.1.1.1  2000/09/16 23:38:08  wex
 * Initial revision
 *
 */

#include "tui_internal.h"

#include <string.h>

#ifdef TUI_XWIN
#define XK_LATIN1
#define XK_MISCELLANY
#include <X11/keysymdef.h>
static void tui_window_key(TUI_WIDGET *window, XKeyEvent *event);
#endif




TUI_WIDGET *
TUI_create_window(TUI *tui, char *title, int x, int y, int width, int height)
{
	TUI_WIDGET *widget;

#ifdef TUI_XWIN	
	Window window;
	XSizeHints sizehints;

	window = tui_create_xwin_window(tui, DefaultRootWindow(tui->app->display),
        x, y, width, height);
	if (window == 0) {
		return NULL;
	}

	sizehints.flags = USPosition | USSize | PMinSize | PMaxSize;
	sizehints.x = x;
	sizehints.y = y;
	sizehints.width = width;
	sizehints.height = height;
    sizehints.min_width = 300;
    sizehints.min_height = 118;
    sizehints.max_width = 1280;
    sizehints.max_height = 1024;
    
	XSetStandardProperties(tui->app->display, window, title, title, None,
        NULL, 0, &sizehints);
	
    XSelectInput(tui->app->display, window,
		EnterWindowMask | LeaveWindowMask | KeyPressMask | ButtonPressMask |
		ButtonReleaseMask | KeyReleaseMask | StructureNotifyMask |
        PointerMotionMask);
#endif

#ifdef TUI_WIN32
	HWND hwnd;

	hwnd = tui_create_win32_window(tui, x, y, width, height);
	if (hwnd == NULL) {
		return NULL;
	}
    SetWindowLong(hwnd, GWL_USERDATA, (long)tui);
#endif

#ifdef TUI_OSX
    Rect rect;
    WindowRef window;
    ControlHandle control;
    unsigned char buf[255];
    
    rect.top = x;
    rect.left = y;
    rect.bottom = y + height;
    rect.right = x + width;
    
    if (CreateNewWindow(kDocumentWindowClass, 0, &rect, &window) < 0) {
        fprintf(stderr, "Cannot create window\n");
        return NULL;
    }
    sprintf(buf, " %s", title);
    buf[0] = strlen(title);
    SetWTitle(window, buf);
    CreateRootControl(window, &control);
#endif
    
	widget = tui_widget_allocate();
	if (widget == NULL) {
		return NULL;
	}

	widget->tui = tui;
	widget->type = TUI_WINDOW;
	widget->parent = tui->root;

    tui_widget_append_to_list(&tui->root->child, widget);

#ifdef TUI_XWIN	
	widget->window = window;
    widget->key_fp = tui_window_key;
#endif

#ifdef TUI_WIN32
	widget->hwnd = hwnd;
#endif

#ifdef TUI_OSX
    widget->window = window;
    SetWRefCon(window, (SInt32)widget);
#endif
    
	widget->x = x;
	widget->y = y;
	widget->w = width;
	widget->h = height;

	if (title != NULL) {
		widget->text = strdup(title);
	} else {
		widget->text = NULL;
	}

    TUI_WIDGET_set_enabled(widget, 1);

	return widget;
}


void
tui_window_disable_matching_radios(TUI_WIDGET *window, TUI_WIDGET *radio)
{
    TUI_WIDGET *widget;
    
    for (widget = window->child; widget != NULL; widget = widget->next) {
        if (widget == radio) continue;
                
        if (widget->type == TUI_RADIO_BUTTON &&
            widget->callback == radio->callback) {

            if (widget->value) {
                widget->value = 0;
#ifdef TUI_XWIN
                widget->draw_fp(widget);
#endif
#ifdef TUI_WIN32
                CheckDlgButton(window->hwnd, widget->control_id, 0);
#endif                
            }
        }
    }
}


#ifdef TUI_XWIN

static TUI_WIDGET *
find_widget_by_key(TUI_WIDGET *widget, char key)
{
    TUI_WIDGET *tmp;
    
    if (widget->key == key) {
        return widget;
    }

    if (widget->child != NULL) {
        tmp = find_widget_by_key(widget->child, key);
        if (tmp != NULL) return tmp;
    }

    if (widget->next != NULL) {
        tmp = find_widget_by_key(widget->next, key);
        if (tmp != NULL) return tmp;
    }

    return NULL;
}


static TUI_WIDGET *
find_selected_textport(TUI_WIDGET *widget)
{
    for ( ; widget != NULL; widget = widget->next) {
        if (widget->type == TUI_TEXTPORT && widget->selected) return widget;
    }

    return NULL;
}


static TUI_WIDGET *
find_next_textport(TUI_WIDGET *widget)
{
    TUI_WIDGET *cur;
    
    for (cur = widget->next; cur != NULL; cur = cur->next) {
        if (cur->type == TUI_TEXTPORT) return cur;
    }

    for (cur = widget->parent->child; cur != widget; cur = cur->next) {
        if (cur->type == TUI_TEXTPORT) return cur;
    }

    return widget;
}


static void
tui_window_key(TUI_WIDGET *window, XKeyEvent *event)
{
    char buffer[128], key;
    KeySym keysym;
    XComposeStatus status;
    TUI_WIDGET *widget, *textport, *next_textport;
    TUI_WIDGET_INFO info;

    if (event->type != KeyPress) return;
    
    XLookupString(event, buffer, 128, &keysym, &status);

    if (!(event->state & ControlMask)) {
        textport = find_selected_textport(window->child);
        if (textport != NULL) {
            if (keysym == XK_Tab) {
                /* rotate the textport selection */
                next_textport = find_next_textport(textport);
                if (next_textport != textport) {
                    tui_widget_deselect_all_textports(window->child);
                    next_textport->selected = 1;
                    next_textport->draw_fp(next_textport);
                }
            } else {
                /* forward this on to the selected textport, if any */
                textport->key_fp(textport, event);
            }
        }

        return;
    }
    
    key = (char) ((int) keysym - XK_0 + '0');

    widget = find_widget_by_key(window->child, key);

    if (widget != NULL) {
        if (widget->callback != NULL) {
            info.widget = widget;
            info.user_data = widget->user_data;
            widget->callback(&info);
        }
    }
}

#endif  /* TUI_XWIN */


TUI_WIDGET *
TUI_window_get_menubar(TUI_WIDGET *window)
{
    TUI_WIDGET *menubar;

    for (menubar = window->child; menubar != NULL; menubar = menubar->next) {
        if (menubar->type == TUI_MENUBAR) break;
    }

    return menubar;
}


void
TUI_window_set_intercept_callback(TUI_WIDGET *window,
    int (*callback)(TUI_EVENT_INFO *info), void *user_data)
{
    assert(window->type == TUI_WINDOW);
    
    window->intercept = callback;
    window->intercept_user_data = user_data;
}

