/* Copyright (c) 2001, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/zb/zb.h,v 1.7 2002/01/03 05:43:26 wex Exp $
 *
 * $Log: zb.h,v $
 * Revision 1.7  2002/01/03 05:43:26  wex
 * micropolygon interpolation controls
 *
 * Revision 1.6  2001/12/16 00:28:34  wex
 * Fixes to frustum culling
 *
 * Revision 1.5  2001/12/05 10:40:44  wex
 * Displacement, transparency, viewport scaling
 *
 * Revision 1.4  2001/11/12 09:14:12  wex
 * depth of field, shading rate, bug fixes and optimizations
 *
 * Revision 1.3  2001/11/10 06:00:39  wex
 * Support for camera window, supersampling and filtering.
 *
 * Revision 1.2  2001/11/04 08:14:59  wex
 * Changes to support material bindings.
 *
 * Revision 1.1  2001/10/26 15:48:42  wex
 * First working version
 *
 *
 */

#ifndef _ZB_H_
#define _ZB_H_

#include <image.h>
#include <shader.h>

/* error return codes */
#define ZB_ERR_NONE             0
#define ZB_ERR_MEMORY           1
#define ZB_ERR_LIST             2
#define ZB_ERR_IMAGE            3
#define ZB_ERR_FILENAME         4

/* contains the error code if a function returns an error */
extern int ZB_errno;

/* returns a user readable string for error messages */
char *ZB_error_string(void);

typedef struct zb ZB;

typedef enum {
    ZB_FILTER_BOX, ZB_FILTER_TRIANGLE, ZB_FILTER_GAUSSIAN
} ZB_FILTER_TYPE;

ZB *ZB_create(void);
void ZB_destroy(ZB *zb);

int ZB_set_camera_format(ZB *zb, char *format);
void ZB_scale_camera_viewport(ZB *zb, float scale);
void ZB_set_camera_samples(ZB *zb, int supersample_rate);
void ZB_set_camera_w2c(ZB *zb, MATRIX *w2c, MATRIX *w2c_motion);
void ZB_set_filter_kernel(ZB *zb, ZB_FILTER_TYPE type, int r);
void ZB_set_dof(ZB *zb, double focus_dist, double scale_factor,
    double focal_length, double aperture);

/* set to true to use triangle intersections and barycentric interpolation */
void ZB_set_interpolate_mp(ZB *zb, int state);

void ZB_set_camera_transforms(ZB *zb, MATRIX *w2c, MATRIX *w2c_motion,
    MATRIX *c2s);
void ZB_set_camera_viewport(ZB *zb, int xmin, int ymin, int xmax, int ymax);
void ZB_set_camera_dof(ZB *zb, double aperture, double focal_length,
    double focus_distance);

MATRIX *ZB_get_camera_w2s(ZB *zb);

int ZB_add_geometry(ZB *zb, PyObject *geometry, PyObject *material,
    PyObject *displacement, double shading_rate);

int ZB_add_light(ZB *zb, PyObject *light);

int ZB_render(ZB *zb);

IMAGE *ZB_get_image(ZB *zb);
void ZB_print_statistics(ZB *zb);


#endif /* _ZB_H_ */
