/* Copyright (c) 2001, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/zb/zb_camera.h,v 1.7 2001/12/20 22:00:46 wex Exp $
 *
 * $Log: zb_camera.h,v $
 * Revision 1.7  2001/12/20 22:00:46  wex
 * Updates for Win32, fixed some matrix memory leaks
 *
 * Revision 1.6  2001/12/05 10:40:44  wex
 * Displacement, transparency, viewport scaling
 *
 * Revision 1.5  2001/11/16 09:25:19  wex
 * Added get_viewport
 *
 * Revision 1.4  2001/11/12 09:14:12  wex
 * depth of field, shading rate, bug fixes and optimizations
 *
 * Revision 1.3  2001/11/10 06:00:39  wex
 * Support for camera window, supersampling and filtering.
 *
 * Revision 1.2  2001/11/06 06:52:37  wex
 * Motion blur, intersection cleanup
 *
 * Revision 1.1  2001/10/26 15:48:42  wex
 * First working version
 *
 *
 */

#ifndef _ZB_CAMERA_H_
#define _ZB_CAMERA_H_

#include <matrix.h>

#include "zb_internal.h"

ZB_CAMERA *zb_camera_create(void);
void zb_camera_destroy(ZB_CAMERA *camera);
int zb_camera_get_width(ZB_CAMERA *camera);
int zb_camera_get_height(ZB_CAMERA *camera);
void zb_camera_get_viewport(ZB_CAMERA *camera, int viewport[4]);
MATRIX *zb_camera_get_w2c(ZB_CAMERA *camera);
MATRIX *zb_camera_get_w2c_motion(ZB_CAMERA *camera);
MATRIX *zb_camera_get_c2s(ZB_CAMERA *camera);
MATRIX *zb_camera_get_w2s(ZB_CAMERA *camera);
MATRIX *zb_camera_get_w2s_motion(ZB_CAMERA *camera);
int zb_camera_get_supersample_rate(ZB_CAMERA *camera);
void zb_camera_get_screen_space_bbox(ZB_CAMERA *camera, MATRIX *l2w,
    MATRIX *l2w_motion, VECTOR *vlmin, VECTOR *vlmax,
    VECTOR *smin, VECTOR *smax);
void zb_camera_get_image_viewport(ZB_CAMERA *camera, int src[4], int dst[4]);

#endif /* _ZB_CAMERA_H_ */
