/* Copyright (c) 2001, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/zb/zb_internal.h,v 1.7 2001/12/20 22:00:46 wex Exp $
 *
 * $Log: zb_internal.h,v $
 * Revision 1.7  2001/12/20 22:00:46  wex
 * Updates for Win32, fixed some matrix memory leaks
 *
 * Revision 1.6  2001/12/16 00:28:34  wex
 * Fixes to frustum culling
 *
 * Revision 1.5  2001/11/13 11:46:27  wex
 * Removed ZB_GRID_DATA, ZB_GEOMETRY_DATA, and tile grid and geometry lists
 *
 * Revision 1.4  2001/11/12 09:14:12  wex
 * depth of field, shading rate, bug fixes and optimizations
 *
 * Revision 1.3  2001/11/10 06:00:39  wex
 * Support for camera window, supersampling and filtering.
 *
 * Revision 1.2  2001/11/01 04:26:59  wex
 * Working with Python scene description and geometry shaders
 *
 * Revision 1.1  2001/10/26 15:48:42  wex
 * First working version
 *
 *
 */

#ifndef _ZB_INTERNAL_H_
#define _ZB_INTERNAL_H_

#include <image.h>
#include <list.h>

#include "zb.h"

typedef struct zb_geometry ZB_GEOMETRY;
typedef struct zb_light ZB_LIGHT;
typedef struct zb_tile ZB_TILE;
typedef struct zb_pixel ZB_PIXEL;
typedef struct zb_camera ZB_CAMERA;
typedef struct zb_grid ZB_GRID;
typedef struct zb_filter ZB_FILTER;
typedef struct zb_dof ZB_DOF;

typedef enum {
    ZB_BBOX_LEFT,
    ZB_BBOX_RIGHT,
    ZB_BBOX_BELOW,
    ZB_BBOX_ABOVE,
    ZB_BBOX_INTERSECT
} ZB_LOCATION;


struct zb {
    PyObject_HEAD
    
    IMAGE *rgba;
    char *filename;
    LIST *geometry_list;
    LIST *tile_list;
    LIST *grid_list;
    LIST *light_list;
    
    ZB_CAMERA *camera;
    ZB_FILTER *filter;
    ZB_DOF *dof;
    
    /* statistics */
    int nonempty_tiles;
    int captured_grids;
    int scanned_grids;
};

ZB_TILE *zb_get_tile(ZB *zb);
ZB_LOCATION zb_compare_viewports(int a0, int a1, int a2, int a3, int b0,
    int b1, int b2, int b3);
int zb_viewport_contained(int a0, int a1, int a2, int a3, int b0, int b1,
    int b2, int b3);

void zb_initialize(ZB *zb);
void zb_delete(ZB *zb);

void zb_stats(ZB *zb);


#endif /* _ZB_INTERNAL_H_ */
