/* Copyright (c) 2001, Daniel Wexler, All Rights Reserved */
/*
 * $Header: /usr/local/cvsroot/flarg/lib/zb/zb_render.c,v 1.11 2001/12/21 07:20:12 wex Exp $
 *
 * $Log: zb_render.c,v $
 * Revision 1.11  2001/12/21 07:20:12  wex
 * Removed memory leaks
 *
 * Revision 1.10  2001/12/20 22:00:46  wex
 * Updates for Win32, fixed some matrix memory leaks
 *
 * Revision 1.9  2001/11/26 21:27:41  wex
 * removed unused variable
 *
 * Revision 1.8  2001/11/16 09:26:15  wex
 * Culling and stat output
 *
 * Revision 1.7  2001/11/13 11:46:27  wex
 * Removed ZB_GRID_DATA, ZB_GEOMETRY_DATA, and tile grid and geometry lists
 *
 * Revision 1.6  2001/11/13 07:47:19  wex
 * Python references and minor fixes
 *
 * Revision 1.5  2001/11/12 09:14:12  wex
 * depth of field, shading rate, bug fixes and optimizations
 *
 * Revision 1.4  2001/11/02 03:17:34  wex
 * Support for Python material and light shaders, updated VECTOR type
 *
 * Revision 1.3  2001/11/01 04:26:59  wex
 * Working with Python scene description and geometry shaders
 *
 * Revision 1.2  2001/10/29 00:50:29  wex
 * Fixed -Wall warnings.
 *
 * Revision 1.1  2001/10/26 15:48:42  wex
 * First working version
 *
 *
 */

#include <stdio.h>

#include "zb_internal.h"
#include "zb_geometry.h"
#include "zb_camera.h"
#include "zb_tile.h"

int geometry_outside_vp_count = 0;

static void clip_geometry_to_screen_window(LIST *geometry_list,
    int viewport[4]);


int
ZB_render(ZB *zb)
{
    ZB_TILE *tile;
    int i, n;
    int viewport[4];
    int status;
    int percent = 0;


    status = 0;

    IMAGE_notify_start_processing(zb->rgba);
    
    if (LIST_is_empty(zb->geometry_list)) {
        fprintf(stderr, "No geometry objects in view\n");
        goto finished;
    }

    zb_camera_get_viewport(zb->camera, viewport);
    clip_geometry_to_screen_window(zb->geometry_list, viewport);
    
    /* sort the geometry for the camera by distance to COP */
    LIST_sort(zb->geometry_list,
        (int (*)(void*, void*, void*))zb_geometry_compare, NULL);

    /* creates tile grid and assigns geometry to each tile touched by bbox */
    if (zb_tile_initialize(zb) < 0) {
        fprintf(stderr, "Cannot create tiles\n");
        status = -1;
        goto finished;
    }

    n = LIST_count(zb->tile_list) - 1;
    i = 0;
    percent = 0;
    
    fprintf(stderr, "Starting...\b\b\b\b\b\b\b\b\b\b\b");
    fflush(stderr);
    
    /* NOTE: should be able to thread later */
    while ((tile = zb_get_tile(zb)) != NULL) {
        if (zb_tile_render(tile) < 0) {
            status = -1;
            goto finished;
        }
        
        LIST_delete(zb->tile_list, tile);
        zb_tile_destroy(tile);

        if (percent < 100.0 * ++i / n + 0.5) {
            percent = 100.0 * i/n + 0.5;
            fprintf(stderr, "%3d%% completed\b\b\b\b\b\b\b\b\b\b\b\b\b\b",
                percent);
            fflush(stderr);
        }
    }

 finished:

    fprintf(stderr, "Done.                  \n");
    fflush(stderr);
    
    IMAGE_notify_stop_processing(zb->rgba);
    
    return status;

}

static void
clip_geometry_to_screen_window(LIST *geometry_list, int viewport[4])
{
    ZB_GEOMETRY *g, *next;

    g = LIST_head(geometry_list);
    while (g != NULL) {
        next = LIST_next(g);
        if (zb_geometry_compare_viewports(g, viewport) != ZB_BBOX_INTERSECT) {
            geometry_outside_vp_count++;
            LIST_delete(geometry_list, g);
            zb_geometry_destroy(g);
        }
        g = next;
    }
}
